package pacman.actors;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.ai.AIManager;
import pacman.game.GameObject;
import pacman.map.Map;
import pacman.map.Path;
import pacman.util.Direction;

/**
 * The Ghost class is the primary enemy in Pacman. Intelligent decisions of
 * ghosts are made by the AIManager class Ghost is a subclass of Actor
 * 
 * @author Ramsey Kant
 */
public class Ghost extends Actor {
    // Movement
    private Path f018;
    private int f118;
    private boolean f218;

    // State
    private boolean f318;
    private boolean f418;
    private boolean f518;

    /**
     * Class constructor
     * 
     * @param v0
     *            Color of the ghost's 'body'
     * @param v1
     *            Reference to the map
     * @param v2
     *            X coordinate to spawn at
     * @param v3
     *            Y coordinate to spawn at
     * @param v4
     *            Set trapped status
     */
    public Ghost(Color v0, Map v1, int v2, int v3, boolean v4) {
        super(GameObject.f416, v0, v1, v2, v3);
        f218 = true;
        f418 = false;
        f318 = v4;
        f518 = false;
    }

    /**
     * Return the fear status of the ghost
     * 
     * @return True if fearful
     */
    public boolean m018() {
        return f418;
    }

    /**
     * Set fear status. AIManager interperates this setting for behavior
     * 
     * @param v5
     *            Fear status, true if fearful
     */
    public void m118(boolean v5) {
        f418 = v5;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if the ghost is currently in the spawn-jail
     */
    public boolean m218() {
        return f318;
    }

    /**
     * Set the current trapped status
     * 
     * @param v6
     *            Trye uf the ghost is in the spawn-jail
     */
    public void m318(boolean v6) {
        f318 = v6;
    }

    /**
     * Flag that is set to true when the path reaches the last possible step
     * 
     * @return True if the AIManager needs to assign a new path
     */
    public boolean m418() {
        return f218;
    }

    /**
     * Update the Path object for the ghost to follow'
     * 
     * @param v7
     *            Path object generated in process() by the AIManager
     * @see AIManager#m20()
     */
    public void m518(Path v7) {
        f118 = 1;
        f018 = v7;
        f218 = false;
    }

    /**
     * Direct's the paint() function to draw the current path of the ghost on
     * the map
     * 
     * @param v8
     *            If true, debug is on and the path will be drawn
     * @see AIManager#setDebugEnabled
     */
    public void m618(boolean v8) {
        f518 = v8;
    }

    /**
     * Run a think cycle for the AI. Major decisions are made by the AIManager
     * (pathing), this just determines movement and screen draw deltas
     * 
     * @see Actor#m516()
     */
    @Override
    public void m516() {
        // Move to the next step
        if (f018 != null && f118 < f018.m03()) {

            // Figure out the direction
            if ((f018.m33(f118) - f1816) < 0) {
                f312 = Direction.f01;
            } else if ((f018.m33(f118) - f1816) > 0) {
                f312 = Direction.f21;
            } else if ((f018.m23(f118) - f1716) > 0) {
                f312 = Direction.f11;
            } else {
                f312 = Direction.f31;
            }

            // Based on the direction, move the screen delta's and the X,Y
            // coordinates if the # of pixels for the cell have been surpassed
            switch (f312) {
                case f01:
                    f712 = 0;
                    f812 = f812 - f912;
                    // If the movement delta has surpassed the number of pixels for
                    // the cell, set him to the map cell he has reached by his movement.
                    if (Math.abs(f812) >= f1916.f25) {
                        f812 = 0;
                        m712(f1716, f1816 - 1);
                        f118++;
                    }
                    break;
                case f11:
                    f712 = f712 + f912;
                    f812 = 0;
                    if (Math.abs(f712) >= f1916.f25) {
                        f712 = 0;
                        m712(f1716 + 1, f1816);
                        f118++;
                    }
                    break;
                case f21:
                    f712 = 0;
                    f812 = f812 + f912;
                    if (Math.abs(f812) >= f1916.f25) {
                        f812 = 0;
                        m712(f1716, f1816 + 1);
                        f118++;
                    }
                    break;
                case f31:
                    f712 = f712 - f912;
                    f812 = 0;
                    if (Math.abs(f712) >= f1916.f25) {
                        f712 = 0;
                        m712(f1716 - 1, f1816);
                        f118++;
                    }
                    break;
                case f51:
                case f41:
                    // do not move
            }
        } else {
            f218 = true;
        }
    }

    /**
     * Draw the ghost
     * 
     * @see GameObject#m616(Graphics2D)
     */
    @Override
    public void m616(Graphics2D v9) {
        final int v10;
		v10 = (int) ((f1916.f25 * f1716) + f712);
        final int v11;
		v11 = (int) ((f1916.f25 * f1816) + f812);

        v9.setColor(f1616);

        // Body
        if (f418) {
            v9.setColor(Color.WHITE);
        }
        v9.fillArc(v10, v11, f1916.f25, f1916.f25, 0, 360);
        v9.fillRect((int) ((f1916.f25 * f1716) + f712), (int) ((f1916.f25 * f1816)
                + (f1916.f25 / 2) + f812), f1916.f25, f1916.f25 / 2);

        // Eyes
        if (f418) {
            v9.setColor(Color.BLACK);
        } else {
            v9.setColor(Color.WHITE);
        }
        v9.fillOval((int) ((f1916.f25 * f1716) + 4 + f712),
                (int) ((f1916.f25 * f1816) + 3 + f812), 8, 10);
        v9.fillOval((int) ((f1916.f25 * f1716) + 12 + f712),
                (int) ((f1916.f25 * f1816) + 3 + f812), 8, 10);

        // Eyeballs
        v9.setColor(Color.BLUE);
        v9.fillOval((int) ((f1916.f25 * f1716) + 7 + f712),
                (int) ((f1916.f25 * f1816) + 6 + f812), 4, 4);
        v9.fillOval((int) ((f1916.f25 * f1716) + 13 + f712),
                (int) ((f1916.f25 * f1816) + 6 + f812), 4, 4);

        // Debug draw path
        if (f518 && f018 != null) {
            int v12;
			for (v12 = 0; v12 < f018.m03(); v12++) {
                final Path.Step v13;
				v13 = f018.m13(v12);
                v9.setColor(f1616);
                v9.drawLine(f1916.f25 * v13.m03(), f1916.f25 * v13.m13(),
                        (f1916.f25 * v13.m03()) + f1916.f25, (f1916.f25 * v13.m13())
                                + f1916.f25);
            }
        }
    }
}
