package pacman.ai;

import java.util.ArrayList;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.game.GameObject;
import pacman.map.Map;
import pacman.map.Path;
import pacman.map.PathFinder;

/**
 * Strategy management behind the AI (Ghost objects)
 * 
 * @author Ramsey Kant
 */
public class AIManager {
    // References
    private Map f00;
    private Player f10;

    // Logic
    private boolean f20;
    private PathFinder f30;
    private final ArrayList<Ghost> f40;
    private long f50;

    /**
     * Class Constructor
     * 
     * @param v0
     *            Reference to the map object being used by the game
     * @param v1
     *            Reference to the player
     * @param v2
     *            Set the debug flag allowing the AI manager to direct ghosts to
     *            exibit diagnostic behavior
     */
    public AIManager(Map v0, Player v1, boolean v2) {
        // Set vars
        f40 = new ArrayList<Ghost>();
        m10(v0, v1);
        f50 = System.currentTimeMillis() + 10000;
        f20 = v2;
    }

    // Getters and Setters

    /**
     * Direct ghosts to display diagnostic information
     * 
     * @param v3
     *            If true, ghosts will enter debug mode
     * @see Ghost#setDebugDrawPath
     */
    public void m00(boolean v3) {
        f20 = v3;
    }

    /**
     * Set the global map and player references. Ghosts being tracked (in the
     * 'ghosts' ArrayList) will be updated
     * 
     * @param v4
     *            Reference to the map
     * @param v5
     *            Reference to the player object
     */
    public void m10(Map v4, Player v5) {
        f40.clear();
        f00 = v4;
        f10 = v5;
        f30 = new PathFinder(v4, 500, false);

        final int v6;
		v6 = f00.m25();
        int v7;
		for (v7 = 0; v7 < v6; v7++) {
            final Actor v8;
			v8 = f00.m115(v7);
            if (v8.m016() == GameObject.f416) {
                f40.add((Ghost) v8);
            }
        }
    }

    /**
     * Run all logic required for AI operation; fear, ghost release, path
     * updates. Ghost act() functions are called here
     */
    public void m20() {
        // Make sure the game is still running and there is a map
        if (f00 == null) {
            return;
        }

        boolean v9;
		v9 = false;
        if (f00.m125().m213()) {
            v9 = true;
        }

        // Release the next ghost
        if (System.currentTimeMillis() > f50) {
            for (final Ghost v10 : f40) {
                if (v10.m218()) {
                    v10.m318(false);
                    v10.m712(13, 11);
                    f50 = System.currentTimeMillis() + 8000;
                    break;
                }
            }
        }

        // Go through a list of all AI on the map
        for (final Ghost v11 : f40) {
            // If a ghost just died, send them to jail
            if (v11.m312()) {
                final int v12;
				v12 = 11;
                final int v13;
				v13 = 13;
                int v14;
				v14 = 0;
                while (!f00.m185(v12 + v14, v13)) {
                    v14++;
                    if (v14 > 4) {
                        break;
                    }
                }

                // Clear path and move to jail
                v11.m518(null);
                v11.m712(v12, v13);
                v11.m318(true);
                v11.m212(false);
            }

            // Any ghost not trapped is given the current fear status
            if (!v11.m218()) {
                // If fear switches from false to true for this ghost, abandon
                // their current (and likely) chase path
                if (!v11.m018() && v9) {
                    v11.m518(null);
                }
                v11.m118(v9);
            } else {
                v11.m118(false);
            }

            // Develop path for ghost
            if (!v11.m218() && v11.m418()) {
                int v15;
				v15 = f10.m316();
                int v16;
				v16 = f10.m416();
                // 45% chance of randomizing a destination, or if they are
                // fearful
                if (v9 || Math.random() < 0.45) {
                    v15 = (int) (Math.random() * f00.m05());
                    v16 = (int) (Math.random() * f00.m15());
                }
                final Path v17;
				v17 = f30.m04(v11, v11.m316(), v11.m416(), v15, v16);
                v11.m518(v17);
            }

            // Run an act()
            v11.m516();

            // If debug is enabled, force ghost to draw it's path
            v11.m618(f20);
        }
    }
}
