package pacman.editor;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.game.GameObject;
import pacman.map.Map;
import pacman.state.StateEditor;

/**
 * The EditorMarker is used by the StateEditor for navigation and selecting
 * tiles on the map EditorMarker is NOT tracked inside the Map EditorMarker is a
 * subclass of GameObject
 * 
 * @author Ramsey Kant
 */
public class EditorMarker extends GameObject {

    /**
     * Class constructor for EditorMarker
     * 
     * @param v0
     *            Color of the marker
     * @param v1
     *            Reference to the map object
     * @param v2
     *            X coordinate to initially place the marker
     * @param v3
     *            Y coordinate to initially place the marker
     */
    public EditorMarker(Color v0, Map v1, int v2, int v3) {
        super(GameObject.f516, v0, v1, v2, v3);
    }

    // Public Methods

    /**
     * Change tile is the EditorMarker's version of Actor's move() method.
     * Called by keyPressed in StateEditor Moves the Marker on the screen
     * 
     * @param v4
     *            Amount to change the current X coordinate by
     * @param v5
     *            Amount to change the current Y coordinate by
     * @see StateEditor#keyPressed(KeyEvent)
     */
    public void m09(int v4, int v5) {
        // Check bounds
        if (f1716 + v4 < 0 || f1816 + v5 < 0 || f1716 + v4 >= f1916.m05()
                || f1816 + v5 >= f1916.m15()) {
            return;
        }

        f1716 += v4;
        f1816 += v5;
    }

    /**
     * EditorMarker has a blank act() method
     * 
     * @see GameObject#m516()
     */
    @Override
    public void m516() {
        // do nothing
    }

    /**
     * EditorMarker appears as a circle around the tile being edited. The color
     * is set in the constructor
     * 
     * @see GameObject#m616(Graphics2D)
     */
    @Override
    public void m616(Graphics2D v6) {
        final int v7;
		v7 = (f1916.f25 * f1716);
        final int v8;
		v8 = (f1916.f25 * f1816);

        v6.setColor(f1616);

        v6.drawOval(v7, v8, f1916.f25, f1916.f25);
    }

}
