package pacman.game;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.actors.Actor;
import pacman.map.Map;
import pacman.state.StateGame;

/**
 * A game object is anything on the pacman grid (wall, cherry, ghost, player).
 * GameObject is the base class of almost everything within the Map
 * 
 * @author Ramsey Kant
 */
public abstract class GameObject {
    // Static type vars
    public static final int f016 = 1;
    public static final int f116 = 2;
    public static final int f216 = 4;
    public static final int f316 = 8;
    public static final int f416 = 16;
    public static final int f516 = 32; // Virtual
    public static final int f616 = 64; // Virtual
    public static final int f716 = 128;

    // Wall types (Walls aren't instanced GameObject's)
    public static final byte f816 = 1;
    public static final byte f916 = 2;
    public static final byte f1016 = 3;
    public static final byte f1116 = 4;
    public static final byte f1216 = 5;
    public static final byte f1316 = 6;
    public static final byte f1416 = 7;

    // Generic object attributes
    protected int f1516;
    protected Color f1616;
    protected int f1716;
    protected int f1816;

    // Outside refereneces
    protected final Map f1916; // Can only be set once. Object only exists within
                             // the map. If the map changes, new objects are
                             // created

    // Getters and Setters

    /**
     * Return the type of the object set in the constructor. See static types
     * defined in GameObject
     * 
     * @return type of object
     */
    public int m016() {
        return f1516;
    }

    /**
     * Grab the current java.awt.Color (base color) the object is being rendered
     * in
     * 
     * @return Base Color of the object
     */
    public Color m116() {
        return f1616;
    }

    /**
     * Set the current base color used when rendering the object
     * 
     * @param v0
     *            java.awt.Color Color of object
     */
    public void m216(Color v0) {
        f1616 = v0;
    }

    /**
     * Grab the current X coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m712(int, int)
     */
    public int m316() {
        return f1716;
    }

    /**
     * Grab the current Y coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m712(int, int)
     */
    public int m416() {
        return f1816;
    }

    // Public & Protected Abstract methods

    /**
     * Class Constructor for a game object
     * 
     * @param v1
     *            Type of game object (see static types above)
     * @param v2
     *            Standard java Color
     * @param v3
     *            Reference to the global Map
     * @param v4
     *            Initial x coordinate
     * @param v5
     *            Initial y coordinate
     */
    public GameObject(int v1, Color v2, Map v3, int v4, int v5) {
        f1516 = v1;
        f1616 = v2;
        f1916 = v3;
        f1716 = v4;
        f1816 = v5;
    }

    /**
     * Perform a "Think" cycle for the Object This includes things like self
     * maintenance and movement
     */
    public abstract void m516();

    /**
     * Draw the object. Subclasses should define how they are to be drawn. This
     * is called in StateGame's logic()
     * 
     * @param v6
     *            The graphics context
     * @see StateGame#m28()
     */
    public abstract void m616(Graphics2D v6);
}
