package pacman.game;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.actors.Player;
import pacman.map.Map;

/**
 * Item objects are GameObject's that can be manipulated by the Player on the
 * map (teleports, dots, powerups, fruit) Item is a subclass of GameObject
 * 
 * @author Ramsey Kant
 */
public class Item extends GameObject {
    // Teleportation vars
    private int f07;
    private int f17;

    /**
     * Class constructor for Item
     * 
     * @param v0
     *            Object type
     * @param v1
     *            Base color of the item
     * @param v2
     *            Reference to the map object
     * @param v3
     *            X coordinate the item will occupy on the map
     * @param v4
     *            Y coordinate the item with occupy on the map
     * @see GameObject
     */
    public Item(int v0, Color v1, Map v2, int v3, int v4) {
        super(v0, v1, v2, v3, v4);

        f07 = 13;
        f17 = 17;
    }

    /**
     * Set the destination coordinates for teleportation. This isn't useful to
     * any item other than a teleport
     * 
     * @param v5
     *            X destination coordinate
     * @param v6
     *            Y destination coordinate
     */
    public void m07(int v5, int v6) {
        f07 = v5;
        f17 = v6;
    }

    /**
     * Retrieve the teleport destination X coordinate
     * 
     * @return X destination coordinate
     * @see Item#m07(int, int)
     */
    public int m17() {
        return f07;
    }

    /**
     * Retrieve the teleport destination Y coordinate
     * 
     * @return Y destination coordinate
     * @see Item#m07(int, int)
     */
    public int m27() {
        return f17;
    }

    /**
     * Called when the item is picked up / used by the player (in the player's
     * act() function) Add point values or trigger powerup modifiers here (using
     * the pl object)
     * 
     * @param v7
     *            Player that uses the item
     * @return True->Destroy the item. False->Keep the item on the map
     * @see Player#m516()
     */
    public boolean m37(Player v7) {
        boolean v8;
		v8 = false;

        // Perform action based on type
        switch (f1516) {
            case f016:
                v7.m013(10);
                v8 = true;
                break;
            case f116:
                v7.m013(50);
                v7.m313(true);
                v8 = true;
                break;
            case f716:
                v7.m712(f07, f17);
                break;
            default:
                break;
        }

        return v8;
    }

    /**
     * Item's have no "think" process. Blank method
     * 
     * @see GameObject#m516()
     */
    @Override
    public void m516() {
        // do nothing
    }

    /**
     * Draw the item based on it's type
     * 
     * @see GameObject#m616(java.awt.Graphics2D)
     */
    @Override
    public void m616(Graphics2D v9) {
        v9.setColor(f1616);

        final int v10;
		v10 = (f1716 * f1916.f25) + f1916.f25 / 2;
        final int v11;
		v11 = (f1816 * f1916.f25) + f1916.f25 / 2;

        // Render item based on type
        switch (f1516) {
            case f016:
                v9.fillArc(v10 - 4, v11 - 4, 8, 8, 0, 360);
                break;
            case f116:
                v9.fillArc(v10 - 8, v11 - 8, 16, 16, 0, 360);
                break;
            case f716:
                v9.fillOval(v10 - 6, v11 - 8, 12, 16);
                break;
            default:
                break;
        }
    }

}
