package pacman.state;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import pacman.game.Game;

/**
 * A State is a mode of the program where input and functionality are radically
 * different from others portions of the program. We can then effectively
 * separate different logical facilities into their own State subclasses
 * 
 * @author Ramsey Kant
 */
public abstract class State implements KeyListener {
    // Game States
    public static final int f08 = 1;
    public static final int f18 = 2;
    public static final int f28 = 4;
    public static final int f38 = 8;
    // public static final int STATE_GAMEOVER = 16;
    public static final int f48 = 32;
    public static final int f58 = 64;

    protected Game f68;

    /**
     * Class Constructor
     * 
     * @param v0
     *            Reference to the game
     */
    public State(Game v0) {
        f68 = v0;
        m18();
    }

    /**
     * Return the reference to the game object
     * 
     * @return Reference to the game object
     */
    public Game m08() {
        return f68;
    }

    /**
     * Start or reset the state
     * 
     * Can be called either by the Supervisor or the state itself
     */
    public abstract void m18();

    /**
     * Primary logic function called in the mainThreadLoop
     * 
     * Called only by the Supervisor
     */
    public abstract void m28();

    /**
     * Signals the state to terminate. Any final updates should be performed
     * here THIS IS ONLY CALLED INSIDE CHANGESTATE() - DO NOT CALL THIS ANYWHERE
     * ELSE
     */
    public abstract void m38();

    /*
     * Human Input default
     */

    @Override
    public void keyReleased(KeyEvent v1) {
        // do nothing
    }

    @Override
    public void keyTyped(KeyEvent v2) {
        // Esc
        switch (v2.getKeyChar()) {
            case 27:
                f68.m720(f58);
                break;
            default:
                break;
        }
    }
}