package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.ai.AIManager;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;
import pacman.util.Direction;

/**
 * StateGame is a mode of the program where the user can play the game of Pacman
 * various maps StateGame is a subclass of State
 * 
 * @author Ramsey Kant
 */
public class StateGame extends State {

    // Logic object references
    private Player f015;
    private Map f115;
    private AIManager f215;

    // Game vars
    private String f315;
    private int f415;
    private int f515; // Overall score for the game session. The player
                              // object score is only the score for that life /
                              // level
    private int f615;
    private boolean f715;
    private long f815;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f915;
    private int f1015;

    /**
     * StateGame Constructor
     * 
     * @param v0
     *            Reference to the game supervisor
     */
    public StateGame(Game v0) {
        super(v0);
    }

    /**
     * Get the current session score. A session score is the total score of the
     * entire 'game' contained by the limited number of lives This is compounded
     * at game over and in win()
     * 
     * @see StateGame#m215()
     * @see StateGame#m315()
     */
    public int m015() {
        return f515;
    }

    // Public Methods

    /**
     * Reset the state of the game entirely (level 1)
     * 
     * @see State#m18()
     */
    @Override
    public void m18() {
        // Set game vars
        f315 = f68.m320();
        f415 = 0;
        f515 = 0;
        f615 = 99;
        f815 = 0;

        // Respawn (start level 1)
        m115(true);
    }

    /**
     * Respawns the player after a death or win. Similar to reset() except
     * running vars are saved
     * 
     * @param v1
     *            Moves to the next level and corresponding map
     */
    public void m115(boolean v1) {
        f715 = true;
        f815 = System.currentTimeMillis() + 3000;

        // If we're jumping to the next level, reset everything
        if (v1) {
            f415++;

            // Force previous references out of scope
            f015 = null;
            f115 = null;
            f215 = null;

            // Setup the game map
            f68.m220().setBackground(Color.BLACK);
            f115 = new Map(f315, 0.75);
            f915 = f115.m05();
            f1015 = f115.m15();

            // Spawn the player
            f015 = f115.m125();

            // Setup AI
            f215 = new AIManager(f115, f015, f68.m520());

            // Slighly increase the game speed

        } else { // Player died, reset the map
            final int v2;
			v2 = f115.m25();
            int v3;
			for (v3 = 0; v3 < v2; v3++) {
                final Actor v4;
				v4 = f115.m115(v3);
                if (v4 != null) {
                    v4.m712(v4.m012(), v4.m112());
                    v4.m212(false);
                    if (v4.m016() == GameObject.f416) {
                        ((Ghost) v4).m518(null);
                    }
                }
            }
        }
    }

    /**
     * Main game logic for rendering and processing. Called by mainThreadLoop
     * 
     * @see Game#m820()
     * @see State#m28()
     */
    @Override
    public void m28() {
        if (f115 == null) {
            return;
        }

        final Graphics2D v5;
		v5 = f68.m220();

        // Offset the buffer so object's arent clipped by the window borders
        v5.translate(10, 30);

        // Paint right UI with lives remaining, score, highscore etc
        v5.setColor(Color.WHITE);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("PACMAN by Ramsey Kant", 680, 50);
        v5.drawString("Score: " + f015.m113(), 750, 100);
        v5.drawString("Total: " + f515, 750, 150);
        v5.drawString("Lives: " + f615, 750, 200);
        v5.drawString("Level: " + f415, 750, 250);

        // Execute game logic for all entites on the map
        if (!f715) {
            f215.m20();
            f015.m516();
        }

        // Check for player death. End the round if the player is dead
        if (f015.m312()) {
            m315();
            return;
        }

        // Check for a win (all dots collected)
        if (f115.m55() <= 0) {
            m215();
            return;
        }

        Item v6;
		v6 = null;
        int v7;
		for (v7 = 0; v7 < f915; v7++) {
            int v8;
			for (v8 = 0; v8 < f1015; v8++) {
                final byte v9;
				v9 = f115.m95(v7, v8);

                // Switch based on wall type and paint
                v5.setColor(Color.BLUE);
                switch (v9) {
                    case 0:
                        // Nothing
                        break;
                    case GameObject.f816:
                        // Vertical wall, no edges
                        v5.fillRoundRect(v7 * f115.f25 + f115.f45,
                                v8 * f115.f25,
                                f115.f35,
                                f115.f25, 0, 0); // 2x+12 = map.CELL_SIZE.
                                                      // x = 10
                        break;
                    case GameObject.f916:
                        // Horizontal wall, no edges
                        v5.fillRoundRect(v7 * f115.f25,
                                v8 * f115.f25 + f115.f45,
                                f115.f25,
                                f115.f35, 0, 0);
                        break;
                    case GameObject.f1016:
                        v5.fillRoundRect(v7 * f115.f25 + (f115.f25 / 2),
                                v8 * f115.f25 + f115.f45,
                                f115.f25 / 2,
                                f115.f35, 0, 0);
                        v5.fillRoundRect(v7 * f115.f25 + f115.f45,
                                v8 * f115.f25 + (f115.f25 / 2),
                                f115.f35,
                                f115.f25 / 2, 0, 0);
                        break;
                    case GameObject.f1116:
                        v5.fillRoundRect(v7 * f115.f25,
                                v8 * f115.f25 + f115.f45,
                                f115.f25 / 2,
                                f115.f35, 0, 0);
                        v5.fillRoundRect(v7 * f115.f25 + f115.f45,
                                v8 * f115.f25
                                + (f115.f25 / 2),
                                f115.f35, f115.f25 / 2, 0, 0);
                        break;
                    case GameObject.f1216:
                        v5.fillRoundRect(v7 * f115.f25 + (f115.f25 / 2), v8 * f115.f25
                                + f115.f45, f115.f25 / 2, f115.f35, 0, 0); // hori
                        v5.fillRoundRect(v7 * f115.f25 + f115.f45, v8 * f115.f25, 12,
                                f115.f25 / 2, 0, 0); // vert
                        break;
                    case GameObject.f1316:
                        v5.fillRoundRect(v7 * f115.f25, v8 * f115.f25 + f115.f45,
                                f115.f25 / 2, f115.f35, 0, 0); // hori
                        v5.fillRoundRect(v7 * f115.f25 + f115.f45, v8 * f115.f25, f115.f35,
                                f115.f25 / 2, 0, 0); // vert
                        break;
                    case GameObject.f1416:
                        v5.setColor(Color.PINK);
                        v5.fillRoundRect(v7 * f115.f25, v8 * f115.f25 + f115.f45, f115.f25,
                                f115.f35 / 2, 0, 0);
                        break;
                    default:
                        break;
                }

                // Paint any of the items here
                v6 = f115.m105(v7, v8);
                if (v6 != null) {
                    v6.m616(v5);
                }
            }
        }

        final int v10;
		v10 = f115.m25();
        int v11;
		for (v11 = 0; v11 < v10; v11++) {
            final Actor v12;
			v12 = f115.m115(v11);
            if (v12 != null) {
                v12.m616(v5);
            }
        }

        // Debug
        if (f68.m520()) {
            v5.setColor(Color.RED);
            v5.drawString("DEBUG ON", 750, 650);
            /*
             * // Paint gridline overlay for(int i = 0; i < mapWidth; i++)
             * g.drawLine(i*map.CELL_SIZE, 0, i*map.CELL_SIZE,
             * mapHeight*map.CELL_SIZE); for(int i = 0; i < mapHeight; i++)
             * g.drawLine(0, i*map.CELL_SIZE, mapWidth*map.CELL_SIZE,
             * i*map.CELL_SIZE);
             */

            // Player X,Y coordinates bottom right
            v5.drawString("positionX: " + f015.m316(), 750, 675);
            v5.drawString("positionY: " + f015.m416(), 750, 700);
        }

        // Check for game pause and print pause status
        if (f715) {
            v5.setColor(Color.RED);
            v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
            v5.drawString("PAUSED", 750, 500);
            if (f815 > System.currentTimeMillis()) {
                v5.drawString(
                        "Pause ends in..." + ((f815 - System.currentTimeMillis()) / 1000),
                        750, 550);
            }
            if (f815 != 0 && System.currentTimeMillis() > f815) {
                f815 = 0;
                f715 = false;
            }
            return;
        }
    }

    /**
     * Player has died or the Supervisor has decided to change the state
     * abruptly
     * 
     * @see State#m38()
     */
    @Override
    public void m38() {
        // Cleanup
        f015 = null;
        f115 = null;
    }

    /**
     * Player has won, move to the next level Called by logic()
     * 
     * @see StateGame#m28()
     */
    public void m215() {
        f515 += f015.m113();

        m115(true);
    }

    /**
     * Player has died, reset() if lives remain. Otherwise, request a state
     * change thereby end()ing this state Called by logic()
     * 
     * @see StateGame#m28()
     */
    public void m315() {
        f615--;

        if (f615 > 0) {
            m115(false);
        } else {
            if (f415 == 1) {
                f515 = f015.m113(); // win() never called, so
                                                  // score is the 1st level
                                                  // score
            }
            f68.m720(State.f18);
        }
    }

    /**
     * Start automove in certain direction
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v13) {
        if (f015 == null) {
            return;
        }

        switch (v13.getKeyCode()) {
            case KeyEvent.VK_UP:
                f015.m612(Direction.f01);
                break;
            case KeyEvent.VK_RIGHT:
                f015.m612(Direction.f11);
                break;
            case KeyEvent.VK_DOWN:
                f015.m612(Direction.f21);
                break;
            case KeyEvent.VK_LEFT:
                f015.m612(Direction.f31);
                break;
            case KeyEvent.VK_SPACE:
                f015.m612(Direction.f51);
                break;
            case KeyEvent.VK_P:
                // Don't interupt system pauses
                if (f815 < System.currentTimeMillis()) {
                    f715 = !f715;
                }
                break;
            case KeyEvent.VK_V:
                f68.m620();
                // AI debug
                f215.m00(f68.m520());
                break;
            case KeyEvent.VK_0:
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }
}
