package pacman.actors;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;
import pacman.util.Direction;

/**
 * Player (pacman) is the object controlled by the human playing the game Player
 * is a subclass of Actor
 * 
 * @author Ramsey Kant
 */
public class Player extends Actor {
    // State
    private int f013; // Current score - Only valid for the current life /
                          // level. StateGame will pull this on death or on
                          // level change
    private boolean f113; // Powered up
    private long f213;

    /**
     * Class Constructor for Player
     * 
     * @param v0
     *            Reference to the map object
     * @param v1
     *            X coordiante to spawn the player at
     * @param v2
     *            Y coordinate to spawn the player at
     */
    public Player(Map v0, int v1, int v2) {
        super(f316, Color.yellow, v0, v1, v2);

        // State
        f013 = 0;
        f113 = false;
        f213 = 0;
    }

    // Getters and Setters

    /**
     * Increment score by amount. The is the current level score, not the entire
     * session score This function is typically called inside an Item's use()
     * function when the player picks up an item like a dot
     * 
     * @param v3
     *            Amount to increment
     */
    public void m013(int v3) {
        f013 += v3;
    }

    /**
     * Get the current level score of the player
     * 
     * @return the score
     */
    public int m113() {
        return f013;
    }

    /**
     * Returns the isPowered flag which determines whether or not the player is
     * powered up and invicible to ghosts
     * 
     * @return True if the player is powered up
     */
    public boolean m213() {
        return f113;
    }

    /**
     * Set powered up state and start the expirtation time for when the powerup
     * wears off
     * 
     * @param v4
     *            True if powered up, false if otherwise
     * @see Player#m213()
     */
    public void m313(boolean v4) {
        f113 = v4;
        // If powered up, start the timer and increase speed temporarily
        if (f113) {
            f213 = System.currentTimeMillis() + 10000;
        }
    }

    /**
     * Player act() method This should evaluate if there is: - a collission with
     * a ghost and how to handle that interaction - a dot or cherry being eaten
     * (call use() on the item) - a next movement
     */
    @Override
    public void m516() {
        final Actor v5;
		v5 = f1916.m135(f1716, f1816, true);
        if (v5 != null && v5.m016() == GameObject.f416) {
            // Notify the State of the loss if pacman isn't powered up
            if (!f113) {
                m212(true);
                return;
            } else {
                v5.m212(true);
            }
        }

        // Check for powerup expire
        if (System.currentTimeMillis() > f213) {
            m313(false);
        }

        boolean v6;
		v6 = false;
        final Item v7;
		v7 = f1916.m105(f1716, f1816);
        if (v7 != null) {
            v6 = v7.m37(this);
        }

        // Update the item's state in the map (remove if itemDestroy is true)
        if (v6) {
            f1916.m155(f1716, f1816);
        }

        final Direction v8;
		v8 = f412.m014();
        if (v8 != Direction.f41) {
            if (f1916.m205(this, v8)) {
                f312 = v8;
            }
        }

        // Based on the direction, increment the movement delta and set the
        // appropriate orientation
        // The delta's represent the screen position (in pixels) since the last
        // official change in position on the grid
        // When a delta in a certain direction passes the CELL_SIZE, the object
        // can change position in the map grid. This makes for smooth
        // transitions between tiles
        switch (f312) {
            case f01:
                // Move in the direction only if the next map cell in this
                // direction is reachable (not occupied by a wall)
                if (f1916.m195(this, f1716, f1816 - 1)) {
                    f712 = 0;
                    f812 -= 0 - f912;
                    // If the movement delta has surpassed the number of pixels for
                    // the cell, set him to the map cell he has reached by his movement
                    if (Math.abs(f812) >= f1916.f25) {
                        f812 = 0;
                        m712(f1716, f1816 - 1);
                    }
                }
                f612 = 90;
                break;
            case f11:
                if (f1916.m195(this, f1716 + 1, f1816)) {
                    f712 += 0 + f912;
                    f812 = 0;
                    if (Math.abs(f712) >= f1916.f25) {
                        f712 = 0;
                        m712(f1716 + 1, f1816);
                    }
                }
                f612 = 0;
                break;
            case f21:
                if (f1916.m195(this, f1716, f1816 + 1)) {
                    f712 = 0;
                    f812 += 0 + f912;
                    if (Math.abs(f812) >= f1916.f25) {
                        f812 = 0;
                        m712(f1716, f1816 + 1);
                    }
                }
                f612 = -90;
                break;
            case f31:
                if (f1916.m195(this, f1716 - 1, f1816)) {
                    f712 -= 0 - f912;
                    f812 = 0;
                    if (Math.abs(f712) >= f1916.f25) {
                        f712 = 0;
                        m712(f1716 - 1, f1816);
                    }
                }
                f612 = 180;
                break;
            case f51:
            case f41:
                // do not move
        }
    }

    /**
     * Draw & animate pacman
     * 
     * @param v9
     *            The graphics context
     * @see Actor#m516()
     */
    @Override
    public void m616(Graphics2D v9) {
        final int v10;
		v10 = (int) ((f1916.f25 * f1716) + f712);
        final int v11;
		v11 = (int) ((f1916.f25 * f1816) + f812);

        v9.setColor(f1616);

        // Animate Pacman's mouth
        // When the player is half-way through a tile, close the flap. Open it
        // back up when the flap clears a tile.
        // This essentially creates an eating animation
        if ((Math.abs(f712) >= f1916.f25 / 2) || Math.abs(f812) >= f1916.f25 / 2) {
            v9.fillArc(v10, v11, f1916.f25, f1916.f25, 0 + f612, 360); // flap
                                                                                           // closed
        } else {
            v9.fillArc(v10, v11, f1916.f25, f1916.f25, 35 + f612, 270);
        }
    }
}
