package pacman.actors;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.ai.AIManager;
import pacman.game.GameObject;
import pacman.map.Map;
import pacman.map.Path;
import pacman.util.Direction;

/**
 * The Ghost class is the primary enemy in Pacman. Intelligent decisions of
 * ghosts are made by the AIManager class Ghost is a subclass of Actor
 * 
 * @author Ramsey Kant
 */
public class Ghost extends Actor {
    // Movement
    private Path f017;
    private int f117;
    private boolean f217;

    // State
    private boolean f317;
    private boolean f417;
    private boolean f517;

    /**
     * Class constructor
     * 
     * @param v0
     *            Color of the ghost's 'body'
     * @param v1
     *            Reference to the map
     * @param v2
     *            X coordinate to spawn at
     * @param v3
     *            Y coordinate to spawn at
     * @param v4
     *            Set trapped status
     */
    public Ghost(Color v0, Map v1, int v2, int v3, boolean v4) {
        super(GameObject.f410, v0, v1, v2, v3);
        f217 = true;
        f417 = false;
        f317 = v4;
        f517 = false;
    }

    /**
     * Return the fear status of the ghost
     * 
     * @return True if fearful
     */
    public boolean m017() {
        return f417;
    }

    /**
     * Set fear status. AIManager interperates this setting for behavior
     * 
     * @param v5
     *            Fear status, true if fearful
     */
    public void m117(boolean v5) {
        f417 = v5;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if the ghost is currently in the spawn-jail
     */
    public boolean m217() {
        return f317;
    }

    /**
     * Set the current trapped status
     * 
     * @param v6
     *            Trye uf the ghost is in the spawn-jail
     */
    public void m317(boolean v6) {
        f317 = v6;
    }

    /**
     * Flag that is set to true when the path reaches the last possible step
     * 
     * @return True if the AIManager needs to assign a new path
     */
    public boolean m417() {
        return f217;
    }

    /**
     * Update the Path object for the ghost to follow'
     * 
     * @param v7
     *            Path object generated in process() by the AIManager
     * @see AIManager#m27()
     */
    public void m517(Path v7) {
        f117 = 1;
        f017 = v7;
        f217 = false;
    }

    /**
     * Direct's the paint() function to draw the current path of the ghost on
     * the map
     * 
     * @param v8
     *            If true, debug is on and the path will be drawn
     * @see AIManager#setDebugEnabled
     */
    public void m617(boolean v8) {
        f517 = v8;
    }

    /**
     * Run a think cycle for the AI. Major decisions are made by the AIManager
     * (pathing), this just determines movement and screen draw deltas
     * 
     * @see Actor#m510()
     */
    @Override
    public void m510() {
        // Move to the next step
        if (f017 != null && f117 < f017.m013()) {

            // Figure out the direction
            if ((f017.m313(f117) - f1810) < 0) {
                f39 = Direction.f00;
            } else if ((f017.m313(f117) - f1810) > 0) {
                f39 = Direction.f20;
            } else if ((f017.m213(f117) - f1710) > 0) {
                f39 = Direction.f10;
            } else {
                f39 = Direction.f30;
            }

            // Based on the direction, move the screen delta's and the X,Y
            // coordinates if the # of pixels for the cell have been surpassed
            switch (f39) {
                case f00:
                    f79 = 0;
                    f89 -= 0 - f99;
                    // If the movement delta has surpassed the number of pixels for
                    // the cell, set him to the map cell he has reached by his movement.
                    if (Math.abs(f89) >= f1910.f25) {
                        f89 = 0;
                        m79(f1710, f1810 - 1);
                        f117++;
                    }
                    break;
                case f10:
                    f79 += 0 + f99;
                    f89 = 0;
                    if (Math.abs(f79) >= f1910.f25) {
                        f79 = 0;
                        m79(f1710 + 1, f1810);
                        f117++;
                    }
                    break;
                case f20:
                    f79 = 0;
                    f89 += 0 + f99;
                    if (Math.abs(f89) >= f1910.f25) {
                        f89 = 0;
                        m79(f1710, f1810 + 1);
                        f117++;
                    }
                    break;
                case f30:
                    f79 -= 0 - f99;
                    f89 = 0;
                    if (Math.abs(f79) >= f1910.f25) {
                        f79 = 0;
                        m79(f1710 - 1, f1810);
                        f117++;
                    }
                    break;
                case f50:
                case f40:
                    // do not move
            }
        } else {
            f217 = true;
        }
    }

    /**
     * Draw the ghost
     * 
     * @see GameObject#m610(Graphics2D)
     */
    @Override
    public void m610(Graphics2D v9) {
        final int v10;
		v10 = (int) ((f1910.f25 * f1710) + f79);
        final int v11;
		v11 = (int) ((f1910.f25 * f1810) + f89);

        v9.setColor(f1610);

        // Body
        if (f417) {
            v9.setColor(Color.WHITE);
        }
        v9.fillArc(v10, v11, f1910.f25, f1910.f25, 0, 360);
        v9.fillRect((int) ((f1910.f25 * f1710) + f79), (int) ((f1910.f25 * f1810)
                + (f1910.f25 / 2) + f89), f1910.f25, f1910.f25 / 2);

        // Eyes
        if (f417) {
            v9.setColor(Color.BLACK);
        } else {
            v9.setColor(Color.WHITE);
        }
        v9.fillOval((int) ((f1910.f25 * f1710) + 4 + f79),
                (int) ((f1910.f25 * f1810) + 3 + f89), 8, 10);
        v9.fillOval((int) ((f1910.f25 * f1710) + 12 + f79),
                (int) ((f1910.f25 * f1810) + 3 + f89), 8, 10);

        // Eyeballs
        v9.setColor(Color.BLUE);
        v9.fillOval((int) ((f1910.f25 * f1710) + 7 + f79),
                (int) ((f1910.f25 * f1810) + 6 + f89), 4, 4);
        v9.fillOval((int) ((f1910.f25 * f1710) + 13 + f79),
                (int) ((f1910.f25 * f1810) + 6 + f89), 4, 4);

        // Debug draw path
        if (f517 && f017 != null) {
            int v12;
			v12 = 0;
			while (v12 < f017.m013()) {
				final Path.Step v13;
				v13 = f017.m113(v12);
				v9.setColor(f1610);
				v9.drawLine(f1910.f25 * v13.m013(), f1910.f25 * v13.m113(),
						(f1910.f25 * v13.m013()) + f1910.f25,
						(f1910.f25 * v13.m113()) + f1910.f25);
				v12++;
			}
        }
    }
}
