package pacman.actors;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;
import pacman.util.Direction;

/**
 * Player (pacman) is the object controlled by the human playing the game Player
 * is a subclass of Actor
 * 
 * @author Ramsey Kant
 */
public class Player extends Actor {
    // State
    private int f015; // Current score - Only valid for the current life /
                          // level. StateGame will pull this on death or on
                          // level change
    private boolean f115; // Powered up
    private long f215;

    /**
     * Class Constructor for Player
     * 
     * @param v0
     *            Reference to the map object
     * @param v1
     *            X coordiante to spawn the player at
     * @param v2
     *            Y coordinate to spawn the player at
     */
    public Player(Map v0, int v1, int v2) {
        super(f310, Color.yellow, v0, v1, v2);

        // State
        f015 = 0;
        f115 = false;
        f215 = 0;
    }

    // Getters and Setters

    /**
     * Increment score by amount. The is the current level score, not the entire
     * session score This function is typically called inside an Item's use()
     * function when the player picks up an item like a dot
     * 
     * @param v3
     *            Amount to increment
     */
    public void m015(int v3) {
        f015 += v3;
    }

    /**
     * Get the current level score of the player
     * 
     * @return the score
     */
    public int m115() {
        return f015;
    }

    /**
     * Returns the isPowered flag which determines whether or not the player is
     * powered up and invicible to ghosts
     * 
     * @return True if the player is powered up
     */
    public boolean m215() {
        return f115;
    }

    /**
     * Set powered up state and start the expirtation time for when the powerup
     * wears off
     * 
     * @param v4
     *            True if powered up, false if otherwise
     * @see Player#m215()
     */
    public void m315(boolean v4) {
        f115 = v4;
        // If powered up, start the timer and increase speed temporarily
        if (f115) {
            f215 = System.currentTimeMillis() + 10000;
        }
    }

    /**
     * Player act() method This should evaluate if there is: - a collission with
     * a ghost and how to handle that interaction - a dot or cherry being eaten
     * (call use() on the item) - a next movement
     */
    @Override
    public void m510() {
        final Actor v5;
		v5 = f1910.m135(f1710, f1810, true);
        if (v5 != null && v5.m010() == GameObject.f410) {
            // Notify the State of the loss if pacman isn't powered up
            if (!f115) {
                m29(true);
                return;
            } else {
                v5.m29(true);
            }
        }

        // Check for powerup expire
        if (System.currentTimeMillis() > f215) {
            m315(false);
        }

        boolean v6;
		v6 = false;
        final Item v7;
		v7 = f1910.m105(f1710, f1810);
        if (v7 != null) {
            v6 = v7.m36(this);
        }

        // Update the item's state in the map (remove if itemDestroy is true)
        if (v6) {
            f1910.m155(f1710, f1810);
        }

        final Direction v8;
		v8 = f49.m011();
        if (v8 != Direction.f40) {
            if (f1910.m205(this, v8)) {
                f39 = v8;
            }
        }

        // Based on the direction, increment the movement delta and set the
        // appropriate orientation
        // The delta's represent the screen position (in pixels) since the last
        // official change in position on the grid
        // When a delta in a certain direction passes the CELL_SIZE, the object
        // can change position in the map grid. This makes for smooth
        // transitions between tiles
        switch (f39) {
            case f00:
                // Move in the direction only if the next map cell in this
                // direction is reachable (not occupied by a wall)
                if (f1910.m195(this, f1710, f1810 - 1)) {
                    f79 = 0;
                    f89 -= 0 - f99;
                    // If the movement delta has surpassed the number of pixels for
                    // the cell, set him to the map cell he has reached by his movement
                    if (Math.abs(f89) >= f1910.f25) {
                        f89 = 0;
                        m79(f1710, f1810 - 1);
                    }
                }
                f69 = 90;
                break;
            case f10:
                if (f1910.m195(this, f1710 + 1, f1810)) {
                    f79 += 0 + f99;
                    f89 = 0;
                    if (Math.abs(f79) >= f1910.f25) {
                        f79 = 0;
                        m79(f1710 + 1, f1810);
                    }
                }
                f69 = 0;
                break;
            case f20:
                if (f1910.m195(this, f1710, f1810 + 1)) {
                    f79 = 0;
                    f89 += 0 + f99;
                    if (Math.abs(f89) >= f1910.f25) {
                        f89 = 0;
                        m79(f1710, f1810 + 1);
                    }
                }
                f69 = -90;
                break;
            case f30:
                if (f1910.m195(this, f1710 - 1, f1810)) {
                    f79 -= 0 - f99;
                    f89 = 0;
                    if (Math.abs(f79) >= f1910.f25) {
                        f79 = 0;
                        m79(f1710 - 1, f1810);
                    }
                }
                f69 = 180;
                break;
            case f50:
            case f40:
                // do not move
        }
    }

    /**
     * Draw & animate pacman
     * 
     * @param v9
     *            The graphics context
     * @see Actor#m510()
     */
    @Override
    public void m610(Graphics2D v9) {
        final int v10;
		v10 = (int) ((f1910.f25 * f1710) + f79);
        final int v11;
		v11 = (int) ((f1910.f25 * f1810) + f89);

        v9.setColor(f1610);

        // Animate Pacman's mouth
        // When the player is half-way through a tile, close the flap. Open it
        // back up when the flap clears a tile.
        // This essentially creates an eating animation
        if ((Math.abs(f79) >= f1910.f25 / 2) || Math.abs(f89) >= f1910.f25 / 2) {
            v9.fillArc(v10, v11, f1910.f25, f1910.f25, 0 + f69, 360); // flap
                                                                                           // closed
        } else {
            v9.fillArc(v10, v11, f1910.f25, f1910.f25, 35 + f69, 270);
        }
    }
}
