package pacman.ai;

import java.util.ArrayList;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.game.GameObject;
import pacman.map.Map;
import pacman.map.Path;
import pacman.map.PathFinder;

/**
 * Strategy management behind the AI (Ghost objects)
 * 
 * @author Ramsey Kant
 */
public class AIManager {
    // References
    private Map f07;
    private Player f17;

    // Logic
    private boolean f27;
    private PathFinder f37;
    private final ArrayList<Ghost> f47;
    private long f57;

    /**
     * Class Constructor
     * 
     * @param v0
     *            Reference to the map object being used by the game
     * @param v1
     *            Reference to the player
     * @param v2
     *            Set the debug flag allowing the AI manager to direct ghosts to
     *            exibit diagnostic behavior
     */
    public AIManager(Map v0, Player v1, boolean v2) {
        // Set vars
        f47 = new ArrayList<Ghost>();
        m17(v0, v1);
        f57 = System.currentTimeMillis() + 10000;
        f27 = v2;
    }

    // Getters and Setters

    /**
     * Direct ghosts to display diagnostic information
     * 
     * @param v3
     *            If true, ghosts will enter debug mode
     * @see Ghost#setDebugDrawPath
     */
    public void m07(boolean v3) {
        f27 = v3;
    }

    /**
     * Set the global map and player references. Ghosts being tracked (in the
     * 'ghosts' ArrayList) will be updated
     * 
     * @param v4
     *            Reference to the map
     * @param v5
     *            Reference to the player object
     */
    public void m17(Map v4, Player v5) {
        f47.clear();
        f07 = v4;
        f17 = v5;
        f37 = new PathFinder(v4, 500, false);

        final int v6;
		v6 = f07.m25();
        int v7;
		v7 = 0;
		while (v7 < v6) {
			final Actor v8;
			v8 = f07.m115(v7);
			if (v8.m010() == GameObject.f410) {
				f47.add((Ghost) v8);
			}
			v7++;
		}
    }

    /**
     * Run all logic required for AI operation; fear, ghost release, path
     * updates. Ghost act() functions are called here
     */
    public void m27() {
        // Make sure the game is still running and there is a map
        if (f07 == null) {
            return;
        }

        boolean v9;
		v9 = false;
        if (f07.m125().m215()) {
            v9 = true;
        }

        // Release the next ghost
        if (System.currentTimeMillis() > f57) {
            for (final Ghost v10 : f47) {
                if (v10.m217()) {
                    v10.m317(false);
                    v10.m79(13, 11);
                    f57 = System.currentTimeMillis() + 8000;
                    break;
                }
            }
        }

        // Go through a list of all AI on the map
        for (final Ghost v11 : f47) {
            // If a ghost just died, send them to jail
            if (v11.m39()) {
                final int v12;
				v12 = 11;
                final int v13;
				v13 = 13;
                int v14;
				v14 = 0;
                for (; !f07.m185(v12 + v14, v13);) {
					v14++;
					if (v14 > 4) {
						break;
					}
				}

                // Clear path and move to jail
                v11.m517(null);
                v11.m79(v12, v13);
                v11.m317(true);
                v11.m29(false);
            }

            // Any ghost not trapped is given the current fear status
            if (!v11.m217()) {
                // If fear switches from false to true for this ghost, abandon
                // their current (and likely) chase path
                if (!v11.m017() && v9) {
                    v11.m517(null);
                }
                v11.m117(v9);
            } else {
                v11.m117(false);
            }

            // Develop path for ghost
            if (!v11.m217() && v11.m417()) {
                int v15;
				v15 = f17.m310();
                int v16;
				v16 = f17.m410();
                // 45% chance of randomizing a destination, or if they are
                // fearful
                if (v9 || Math.random() < 0.45) {
                    v15 = (int) (Math.random() * f07.m05());
                    v16 = (int) (Math.random() * f07.m15());
                }
                final Path v17;
				v17 = f37.m08(v11, v11.m310(), v11.m410(), v15, v16);
                v11.m517(v17);
            }

            // Run an act()
            v11.m510();

            // If debug is enabled, force ghost to draw it's path
            v11.m617(f27);
        }
    }
}
