package pacman.game;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;

import pacman.state.State;
import pacman.state.StateEditor;
import pacman.state.StateGame;
import pacman.state.StateMenu;
import pacman.state.StateScoreboard;

/**
 * The Game Supervisor. This class implements that core program logic, state
 * management, and graphics.
 * 
 * @author Ramsey Kant
 */
public class Game extends Canvas {
    private static final long f03 = 1L;

    // Debug vars
    private boolean f13;

    // Threading
    private boolean f23;

    // Graphics variables
    private Frame f33;
    public final int f43;
    public final int f53;
    private BufferStrategy f63;

    // State
    private int f73;
    private State f83;
    private boolean f93;
    private int f103;
    private String f113;

    /**
     * Class Constructor Set's up graphics and put's game logic into a startup
     * state by calling init()
     * 
     * @param v0
     *            Resolution X
     * @param v1
     *            Resolution Y
     * @see Game#m03()
     */
    public Game(int v0, int v1) {
        // Set resolution settings
        f43 = v0;
        f53 = v1;

        // Init game
        m03();
    }

    /**
     * Startup functionality for the program called by the constructor
     */
    private void m03() {
        // Debug vars
        f13 = false;

        f113 = "test.map";
        f93 = false;

        // Setup the game frame
        f33 = new Frame("Pacman");
        f33.setLayout(null);
        setBounds(0, 0, f43, f53);
        f33.add(this);
        f33.setSize(f43, f53);
        f33.setResizable(false);
        f33.setVisible(true);

        // Set the exit handler with an anonymous class
        f33.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent v2) {
                // Exit main thread
                f23 = false;
            }
        });

        // Setup double buffering
        setIgnoreRepaint(true); // We'll handle repainting
        createBufferStrategy(2);
        f63 = getBufferStrategy();

        f23 = true;
    }

    // Getter and Setter methods

    /**
     * Get the Frame object encapsulating the program
     * 
     * @return The frame
     */
    public Frame m13() {
        return f33;
    }

    /**
     * Get a 'handle' of the current graphics buffer for drawing
     * 
     * @return The Graphics2D buffer
     */
    public Graphics2D m23() {
        return (Graphics2D) f63.getDrawGraphics();
    }

    /**
     * Get the name of the map to be loaded in StateGame
     * 
     * @return Map name (with .map extension)
     */
    public String m33() {
        return f113;
    }

    /**
     * Set the default starting map (set by menu)
     * 
     * @param v3
     *            The name of the map to load (with the .map extension)
     */
    public void m43(String v3) {
        f113 = v3;
    }

    /**
     * Return the current debug setting
     * 
     * @return True if debug setting is on
     * @see Game#m63()
     */
    public boolean m53() {
        return f13;
    }

    /**
     * Toggle debugging. Facilities like AIManager use this flag to display
     * diagnostic information like AI paths
     */
    public void m63() {
        f13 = !f13;
    }

    // Public Methods

    /**
     * Called by other states to safely change currentState. This is done so the
     * currentState's logic can finish
     * 
     * @see Game#m83()
     */
    public void m73(int v4) {
        f103 = v4;
        f93 = true;
    }

    /**
     * The main game loop that handles graphics and game state determination
     */
    public void m83() {
    	final long v5;
		v5 = 20;
    	long v6;
		v6 = 0;
    	
        for (; f23;) {
			if ((v6 + v5) > System.currentTimeMillis()) {
				continue;
			}
			v6 = System.currentTimeMillis();
			if (f93) {
				f93 = false;
				m93(f103);
				continue;
			}
			final Graphics2D v7;
			v7 = m23();
			v7.setColor(Color.black);
			v7.fillRect(0, 0, f43, f53);
			f83.m220();
			v7.dispose();
			f63.show();
			try {
				Thread.sleep(10);
			} catch (InterruptedException v8) {
				v8.printStackTrace();
			}
		}
    }

    // Private Methods

    /**
     * Change the state of the game. Called in mainThreadLogic()
     * 
     * @param v9
     *            The state to set. Must match the static vars above
     * @see Game#m73(int)
     * @see Game#m83()
     */
    private void m93(int v9) {
        // Cleanup for the outgoing state
        if (f83 != null) {
            f33.removeKeyListener(f83);
            removeKeyListener(f83);
            f83.m320();
        }

        // Set the new state type
        f73 = v9;

        // Instance the new state (reset() is called in the construtor)
        switch (f73) {
            case State.f220:
                f83 = new StateGame(this);
                break;
            case State.f120:
                f83 = new StateScoreboard(this);
                /*
                 * StateGame sb = new StateScoreboard(); int newScore = 0;
                 * 
                 * // If the previous state was STATE_GAME, pull the session
                 * score and pass it to the scoreboard if(currentState
                 * instanceof StateGame)
                 * sb.addScore((int)((StateGame)currentState
                 * ).getSessionScore()));
                 * 
                 * currentState = sb;
                 */
                break;
            case State.f420:
                f83 = new StateEditor(this);
                break;
            case State.f020:
                f83 = new StateMenu(this);
                break;
            case State.f520:
                f83 = null;
                f23 = false;
                break;
            default:
                break;
        }

        // Setup input handler and reset()
        if (f83 != null) {
            f33.addKeyListener(f83);
            addKeyListener(f83);
        }
    }
}
