package pacman.game;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.actors.Actor;
import pacman.map.Map;
import pacman.state.StateGame;

/**
 * A game object is anything on the pacman grid (wall, cherry, ghost, player).
 * GameObject is the base class of almost everything within the Map
 * 
 * @author Ramsey Kant
 */
public abstract class GameObject {
    // Static type vars
    public static final int f010 = 1;
    public static final int f110 = 2;
    public static final int f210 = 4;
    public static final int f310 = 8;
    public static final int f410 = 16;
    public static final int f510 = 32; // Virtual
    public static final int f610 = 64; // Virtual
    public static final int f710 = 128;

    // Wall types (Walls aren't instanced GameObject's)
    public static final byte f810 = 1;
    public static final byte f910 = 2;
    public static final byte f1010 = 3;
    public static final byte f1110 = 4;
    public static final byte f1210 = 5;
    public static final byte f1310 = 6;
    public static final byte f1410 = 7;

    // Generic object attributes
    protected int f1510;
    protected Color f1610;
    protected int f1710;
    protected int f1810;

    // Outside refereneces
    protected final Map f1910; // Can only be set once. Object only exists within
                             // the map. If the map changes, new objects are
                             // created

    // Getters and Setters

    /**
     * Return the type of the object set in the constructor. See static types
     * defined in GameObject
     * 
     * @return type of object
     */
    public int m010() {
        return f1510;
    }

    /**
     * Grab the current java.awt.Color (base color) the object is being rendered
     * in
     * 
     * @return Base Color of the object
     */
    public Color m110() {
        return f1610;
    }

    /**
     * Set the current base color used when rendering the object
     * 
     * @param v0
     *            java.awt.Color Color of object
     */
    public void m210(Color v0) {
        f1610 = v0;
    }

    /**
     * Grab the current X coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m79(int, int)
     */
    public int m310() {
        return f1710;
    }

    /**
     * Grab the current Y coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m79(int, int)
     */
    public int m410() {
        return f1810;
    }

    // Public & Protected Abstract methods

    /**
     * Class Constructor for a game object
     * 
     * @param v1
     *            Type of game object (see static types above)
     * @param v2
     *            Standard java Color
     * @param v3
     *            Reference to the global Map
     * @param v4
     *            Initial x coordinate
     * @param v5
     *            Initial y coordinate
     */
    public GameObject(int v1, Color v2, Map v3, int v4, int v5) {
        f1510 = v1;
        f1610 = v2;
        f1910 = v3;
        f1710 = v4;
        f1810 = v5;
    }

    /**
     * Perform a "Think" cycle for the Object This includes things like self
     * maintenance and movement
     */
    public abstract void m510();

    /**
     * Draw the object. Subclasses should define how they are to be drawn. This
     * is called in StateGame's logic()
     * 
     * @param v6
     *            The graphics context
     * @see StateGame#m220()
     */
    public abstract void m610(Graphics2D v6);
}
