package pacman.map;
import java.util.ArrayList;

/**
 * A path determined by some path finding algorithm. A series of steps from the
 * starting location to the target location. This includes a step for the
 * initial location.
 * 
 * @author Kevin Glass
 */
public class Path {
    /** The list of steps building up this path */
    private final ArrayList<Step> f013;

    /**
     * Create an empty path
     */
    public Path() {
        f013 = new ArrayList<Step>();
    }

    /**
     * Get the length of the path, i.e. the number of steps
     * 
     * @return The number of steps in this path
     */
    public int m013() {
        return f013.size();
    }

    /**
     * Get the step at a given index in the path
     * 
     * @param v0
     *            The index of the step to retrieve. Note this should be >= 0
     *            and < getLength();
     * @return The step information, the position on the map.
     */
    public Step m113(int v0) {
        return f013.get(v0);
    }

    /**
     * Get the x coordinate for the step at the given index
     * 
     * @param v1
     *            The index of the step whose x coordinate should be retrieved
     * @return The x coordinate at the step
     */
    public int m213(int v1) {
        return m113(v1).f013;
    }

    /**
     * Get the y coordinate for the step at the given index
     * 
     * @param v2
     *            The index of the step whose y coordinate should be retrieved
     * @return The y coordinate at the step
     */
    public int m313(int v2) {
        return m113(v2).f113;
    }

    /**
     * Append a step to the path.
     * 
     * @param v3
     *            The x coordinate of the new step
     * @param v4
     *            The y coordinate of the new step
     */
    public void m413(int v3, int v4) {
        f013.add(new Step(v3, v4));
    }

    /**
     * Prepend a step to the path.
     * 
     * @param v5
     *            The x coordinate of the new step
     * @param v6
     *            The y coordinate of the new step
     */
    public void m513(int v5, int v6) {
        f013.add(0, new Step(v5, v6));
    }

    /**
     * Check if this path contains the given step
     * 
     * @param v7
     *            The x coordinate of the step to check for
     * @param v8
     *            The y coordinate of the step to check for
     * @return True if the path contains the given step
     */
    public boolean m613(int v7, int v8) {
        return f013.contains(new Step(v7, v8));
    }

    /**
     * A single step within the path
     * 
     * @author Kevin Glass
     */
    public class Step {
        /** The x coordinate at the given step */
        private final int f013;
        /** The y coordinate at the given step */
        private final int f113;

        /**
         * Create a new step
         * 
         * @param v9
         *            The x coordinate of the new step
         * @param v10
         *            The y coordinate of the new step
         */
        public Step(int v9, int v10) {
            this.f013 = v9;
            this.f113 = v10;
        }

        /**
         * Get the x coordinate of the new step
         * 
         * @return The x coodindate of the new step
         */
        public int m013() {
            return f013;
        }

        /**
         * Get the y coordinate of the new step
         * 
         * @return The y coodindate of the new step
         */
        public int m113() {
            return f113;
        }

        /**
         * @see Object#hashCode()
         */
        @Override
        public int hashCode() {
            return f013 * f113;
        }

        /**
         * @see Object#equals(Object)
         */
        @Override
        public boolean equals(Object v11) {
            if (v11 instanceof Step) {
                final Step v12;
				v12 = (Step) v11;

                return (v12.f013 == f013) && (v12.f113 == f113);
            }

            return false;
        }
    }
}
