package pacman.state;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.editor.EditorFrame;
import pacman.editor.EditorMarker;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;

/**
 * The StateEditor is a mode of the program that allows the user to create and
 * modify map files that can be played in StateGame. StateEditor is a subclass
 * of State
 * 
 * @author Ramsey Kant
 */
public class StateEditor extends State {

    // Logic object references
    private final EditorFrame f012;
    private EditorMarker f112;
    private boolean f212;
    private Map f312;

    // Placement variables
    private int f412;
    private byte f512;
    private String f612;
    private boolean f712;
    private int f812;
    private int f912;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f1012;
    private int f1112;

    public StateEditor(Game v0) {
        super(v0);

        // If true, remove all editor helpers like grid lines
        f212 = false;

        // Create the editor toolpane
        f620.m13().setSize(1024, f620.f53);
        f012 = new EditorFrame(this);
        f012.setVisible(true);

        // Defaults
        f412 = GameObject.f610;
        f512 = GameObject.f810;
        f612 = "Blinky";
        f712 = false;
        f812 = 13;
        f912 = 17;
    }

    // Getters and Setters

    /**
     * Set the type of object to be placed by the marker Called by the
     * EditorFrame (Dot, Powerup, Teleport buttons)
     * 
     * @param v1
     *            Type of object (from GameObject statics)
     */
    public void m012(int v1) {
        f412 = v1;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Wall button
     * 
     * @param v2
     *            Type of wall (from GameObject statics)
     */
    public void m112(byte v2) {
        f512 = v2;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Add Ghost button
     * 
     * @param v3
     *            Type of wall (from GameObject statics)
     */
    public void m212(String v3) {
        f612 = v3;
    }

    /**
     * Toggle the trapped status of the next ghost to be added
     * 
     * @param v4
     *            True if trapped in the spawn-jail
     */
    public void m312(boolean v4) {
        f712 = v4;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if new ghosts will be created as trapped
     */
    public boolean m412() {
        return f712;
    }

    /**
     * Set the teleport destination, used to aid a teleport drop Called in
     * EditorFrame by the Teleport add button
     * 
     * @param v5
     *            destination coordinate X of the next teleport
     * @param v6
     *            destination coordinate Y of the next teleport
     */
    public void m512(int v5, int v6) {
        f812 = v5;
        f912 = v6;
    }

    /**
     * Reset the StateEditor objects like the Marker
     * 
     * @see State#m120()
     */
    @Override
    public void m120() {
        // Force previous references out of scope
        f112 = null;
        f312 = null;

        f412 = GameObject.f010;
    }

    /**
     * Setup and render a new blank map
     * 
     * @param v7
     *            The width of the map to be created
     * @param v8
     *            The height of the map to be created
     */
    public void m612(int v7, int v8) {
        // Setup the game map
        f620.m23().setBackground(Color.BLACK);
        f1012 = v7;
        f1112 = v8;
        f312 = new Map(28, 31, 32);

        // Create the marker (but don't put it "in" the map)
        f112 = new EditorMarker(Color.GREEN, f312, 0, 0);
    }

    /**
     * Save the map
     * 
     * @param v9
     */
    public void m712(String v9) {
        f312.m235(System.getProperty("user.dir") + "\\" + v9);
    }

    /**
     * Setup and render a map loaded from the file system
     * 
     * @param v10
     */
    public void m812(String v10) {
        // Setup the game map
        f620.m23().setBackground(Color.BLACK);
        f312 = new Map(System.getProperty("user.dir") + "\\" + v10, 32);
        f1012 = f312.m05();
        f1112 = f312.m15();

        // Create the marker (but don't put it "in" the map)
        f112 = new EditorMarker(Color.GREEN, f312, 0, 0);
    }

    /**
     * Logic of the editor processed here: Rendering, input, and object
     * placement. Called in the mainThreadLoop
     * 
     * @see State#m220()
     */
    @Override
    public void m220() {
        if (f312 == null) {
            return;
        }

        final Graphics2D v11;
		v11 = f620.m23();

        // Offset the buffer so object's arent clipped by the window borders
        v11.translate(10, 30);

        Item v12;
		v12 = null;
        int v13;
		v13 = 0;
		while (v13 < f1012) {
			int v14;
			v14 = 0;
			while (v14 < f1112) {
				final byte v15;
				v15 = f312.m95(v13, v14);
				v11.setColor(Color.BLUE);
				switch (v15) {
				case 0:
					break;
				case GameObject.f810:
					v11.fillRoundRect(v13 * f312.f25 + 10, v14 * f312.f25, 12,
							f312.f25, 0, 0);
					break;
				case GameObject.f910:
					v11.fillRoundRect(v13 * f312.f25, v14 * f312.f25 + 10,
							f312.f25, 12, 0, 0);
					break;
				case GameObject.f1010:
					v11.fillRoundRect(v13 * f312.f25 + (f312.f25 / 2), v14
							* f312.f25 + 10, f312.f25 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f312.f25 + 10, v14 * f312.f25
							+ (f312.f25 / 2), 12, f312.f25 / 2, 0, 0);
					break;
				case GameObject.f1110:
					v11.fillRoundRect(v13 * f312.f25, v14 * f312.f25 + 10,
							f312.f25 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f312.f25 + 10, v14 * f312.f25
							+ (f312.f25 / 2), 12, f312.f25 / 2, 0, 0);
					break;
				case GameObject.f1210:
					v11.fillRoundRect(v13 * f312.f25 + (f312.f25 / 2), v14
							* f312.f25 + 10, f312.f25 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f312.f25 + 10, v14 * f312.f25, 12,
							f312.f25 / 2, 0, 0);
					break;
				case GameObject.f1310:
					v11.fillRoundRect(v13 * f312.f25, v14 * f312.f25 + 10,
							f312.f25 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f312.f25 + 10, v14 * f312.f25, 12,
							f312.f25 / 2, 0, 0);
					break;
				case GameObject.f1410:
					v11.setColor(Color.PINK);
					v11.fillRoundRect(v13 * f312.f25, v14 * f312.f25 + 10,
							f312.f25, 6, 0, 0);
					break;
				default:
					break;
				}
				v12 = f312.m105(v13, v14);
				if (v12 != null) {
					v12.m610(v11);
				}
				v14++;
			}
			v13++;
		}

        final int v16;
		v16 = f312.m25();
        int v17;
		v17 = 0;
		while (v17 < v16) {
			final Actor v18;
			v18 = f312.m115(v17);
			if (v18 != null) {
				v18.m610(v11);
			}
			v17++;
		}

        // Paint the marker
        f112.m610(v11);

        // Paint gridline overlay if in editor view
        if (!f212) {
            v11.setColor(Color.RED);
            int v19;
			v19 = 0;
			while (v19 < f1012) {
				v11.drawLine(v19 * f312.f25, 0, v19 * f312.f25, f1112
						* f312.f25);
				v19++;
			}
            int v20;
			v20 = 0;
			while (v20 < f1112) {
				v11.drawLine(0, v20 * f312.f25, f1012 * f312.f25, v20
						* f312.f25);
				v20++;
			}

            // Player X,Y coordinates bottom right
            v11.drawString("X: " + f112.m310() + ", Y: " + f112.m410(), 900, 700);
        }
    }

    /**
     * Termination of the StateEdtior. Set references stored by the StateEditor
     * as null
     * 
     * @see State#m320()
     */
    @Override
    public void m320() {
        // Cleanup
        f112 = null;
        f312 = null;
    }

    /**
     * Input processing for the Editor
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v21) {
        switch (v21.getKeyCode()) {
            case KeyEvent.VK_UP:
                f112.m014(0, -1);
                break;
            case KeyEvent.VK_RIGHT:
                f112.m014(1, 0);
                break;
            case KeyEvent.VK_DOWN:
                f112.m014(0, +1);
                break;
            case KeyEvent.VK_LEFT:
                f112.m014(-1, 0);
                break;
            case KeyEvent.VK_ENTER:
                if (f112 == null) {
                    return;
                }

                // If not empty, bail
                if (!f312.m185(f112.m310(), f112.m410())) {
                    return;
                }

                switch (f412) {
                    case GameObject.f610:
                        f312.m65(f112.m310(), f112.m410(), f512);
                        break;
                    case GameObject.f010:
                        f312.m75(new Item(GameObject.f010, Color.WHITE, f312,
                                f112.m310(), f112.m410()));
                        break;
                    case GameObject.f110:
                        f312.m75(new Item(GameObject.f110, Color.WHITE, f312, f112
                                .m310(), f112.m410()));
                        break;
                    case GameObject.f410:
                        if (f612.equals("Blinky")) {
                            f312.m85(new Ghost(Color.RED, f312, f112.m310(), f112.m410(),
                                    f712));
                        } else if (f612.equals("Pinky")) {
                            f312.m85(new Ghost(Color.PINK, f312, f112.m310(), f112.m410(),
                                    f712));
                        } else if (f612.equals("Inky")) {
                            f312.m85(new Ghost(Color.CYAN, f312, f112.m310(), f112.m410(),
                                    f712));
                        } else {
                            f312.m85(new Ghost(Color.ORANGE, f312, f112.m310(), f112.m410(),
                                    f712));
                        }
                        break;
                    case GameObject.f310:
					int v22;
					v22 = 0;
					while (v22 < f312.m25()) {
						if (f312.m115(v22).m010() == GameObject.f310) {
							f312.m145(v22);
							v22--;
						}
						v22++;
					}

                        // Add the new player
                        f312.m85(new Player(f312, f112.m310(), f112.m410()));
                        break;
                    case GameObject.f710:
					final Item v23;
					v23 = new Item(GameObject.f710,
							Color.LIGHT_GRAY, f312, f112.m310(), f112.m410());
                        v23.m06(f812, f912);
                        f312.m75(v23);
                        break;

                    default:
                        break;
                }
                break;
            case KeyEvent.VK_DELETE:
                // Delete a placed object. Will reduce excessive memory
                // consumption if the user cant just replace a tile with a new
                // object

                // If empty, bail
                if (f312.m185(f112.m310(), f112.m410())) {
                    return;
                }

                // Remove anything (collidable, actor, or item) at (x,y)
                f312.m165(f112.m310(), f112.m410());
                break;
            case KeyEvent.VK_V:
                f212 = !f212;
                break;
            case KeyEvent.VK_0:
                // editorFrame.setEnabled(false);
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }

}
