package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.ai.AIManager;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;
import pacman.util.Direction;

/**
 * StateGame is a mode of the program where the user can play the game of Pacman
 * various maps StateGame is a subclass of State
 * 
 * @author Ramsey Kant
 */
public class StateGame extends State {

    // Logic object references
    private Player f04;
    private Map f14;
    private AIManager f24;

    // Game vars
    private String f34;
    private int f44;
    private int f54; // Overall score for the game session. The player
                              // object score is only the score for that life /
                              // level
    private int f64;
    private boolean f74;
    private long f84;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f94;
    private int f104;

    /**
     * StateGame Constructor
     * 
     * @param v0
     *            Reference to the game supervisor
     */
    public StateGame(Game v0) {
        super(v0);
    }

    /**
     * Get the current session score. A session score is the total score of the
     * entire 'game' contained by the limited number of lives This is compounded
     * at game over and in win()
     * 
     * @see StateGame#m24()
     * @see StateGame#m34()
     */
    public int m04() {
        return f54;
    }

    // Public Methods

    /**
     * Reset the state of the game entirely (level 1)
     * 
     * @see State#m120()
     */
    @Override
    public void m120() {
        // Set game vars
        f34 = f620.m33();
        f44 = 0;
        f54 = 0;
        f64 = 99;
        f84 = 0;

        // Respawn (start level 1)
        m14(true);
    }

    /**
     * Respawns the player after a death or win. Similar to reset() except
     * running vars are saved
     * 
     * @param v1
     *            Moves to the next level and corresponding map
     */
    public void m14(boolean v1) {
        f74 = true;
        f84 = System.currentTimeMillis() + 3000;

        // If we're jumping to the next level, reset everything
        if (v1) {
            f44++;

            // Force previous references out of scope
            f04 = null;
            f14 = null;
            f24 = null;

            // Setup the game map
            f620.m23().setBackground(Color.BLACK);
            f14 = new Map(f34, 0.75);
            f94 = f14.m05();
            f104 = f14.m15();

            // Spawn the player
            f04 = f14.m125();

            // Setup AI
            f24 = new AIManager(f14, f04, f620.m53());

            // Slighly increase the game speed

        } else { // Player died, reset the map
            final int v2;
			v2 = f14.m25();
            int v3;
			v3 = 0;
			while (v3 < v2) {
				final Actor v4;
				v4 = f14.m115(v3);
				if (v4 != null) {
					v4.m79(v4.m09(), v4.m19());
					v4.m29(false);
					if (v4.m010() == GameObject.f410) {
						((Ghost) v4).m517(null);
					}
				}
				v3++;
			}
        }
    }

    /**
     * Main game logic for rendering and processing. Called by mainThreadLoop
     * 
     * @see Game#m83()
     * @see State#m220()
     */
    @Override
    public void m220() {
        if (f14 == null) {
            return;
        }

        final Graphics2D v5;
		v5 = f620.m23();

        // Offset the buffer so object's arent clipped by the window borders
        v5.translate(10, 30);

        // Paint right UI with lives remaining, score, highscore etc
        v5.setColor(Color.WHITE);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("PACMAN by Ramsey Kant", 680, 50);
        v5.drawString("Score: " + f04.m115(), 750, 100);
        v5.drawString("Total: " + f54, 750, 150);
        v5.drawString("Lives: " + f64, 750, 200);
        v5.drawString("Level: " + f44, 750, 250);

        // Execute game logic for all entites on the map
        if (!f74) {
            f24.m27();
            f04.m510();
        }

        // Check for player death. End the round if the player is dead
        if (f04.m39()) {
            m34();
            return;
        }

        // Check for a win (all dots collected)
        if (f14.m55() <= 0) {
            m24();
            return;
        }

        Item v6;
		v6 = null;
        int v7;
		v7 = 0;
		while (v7 < f94) {
			int v8;
			v8 = 0;
			while (v8 < f104) {
				final byte v9;
				v9 = f14.m95(v7, v8);
				v5.setColor(Color.BLUE);
				switch (v9) {
				case 0:
					break;
				case GameObject.f810:
					v5.fillRoundRect(v7 * f14.f25 + f14.f45, v8 * f14.f25,
							f14.f35, f14.f25, 0, 0);
					break;
				case GameObject.f910:
					v5.fillRoundRect(v7 * f14.f25, v8 * f14.f25 + f14.f45,
							f14.f25, f14.f35, 0, 0);
					break;
				case GameObject.f1010:
					v5.fillRoundRect(v7 * f14.f25 + (f14.f25 / 2), v8 * f14.f25
							+ f14.f45, f14.f25 / 2, f14.f35, 0, 0);
					v5.fillRoundRect(v7 * f14.f25 + f14.f45, v8 * f14.f25
							+ (f14.f25 / 2), f14.f35, f14.f25 / 2, 0, 0);
					break;
				case GameObject.f1110:
					v5.fillRoundRect(v7 * f14.f25, v8 * f14.f25 + f14.f45,
							f14.f25 / 2, f14.f35, 0, 0);
					v5.fillRoundRect(v7 * f14.f25 + f14.f45, v8 * f14.f25
							+ (f14.f25 / 2), f14.f35, f14.f25 / 2, 0, 0);
					break;
				case GameObject.f1210:
					v5.fillRoundRect(v7 * f14.f25 + (f14.f25 / 2), v8 * f14.f25
							+ f14.f45, f14.f25 / 2, f14.f35, 0, 0);
					v5.fillRoundRect(v7 * f14.f25 + f14.f45, v8 * f14.f25, 12,
							f14.f25 / 2, 0, 0);
					break;
				case GameObject.f1310:
					v5.fillRoundRect(v7 * f14.f25, v8 * f14.f25 + f14.f45,
							f14.f25 / 2, f14.f35, 0, 0);
					v5.fillRoundRect(v7 * f14.f25 + f14.f45, v8 * f14.f25,
							f14.f35, f14.f25 / 2, 0, 0);
					break;
				case GameObject.f1410:
					v5.setColor(Color.PINK);
					v5.fillRoundRect(v7 * f14.f25, v8 * f14.f25 + f14.f45,
							f14.f25, f14.f35 / 2, 0, 0);
					break;
				default:
					break;
				}
				v6 = f14.m105(v7, v8);
				if (v6 != null) {
					v6.m610(v5);
				}
				v8++;
			}
			v7++;
		}

        final int v10;
		v10 = f14.m25();
        int v11;
		v11 = 0;
		while (v11 < v10) {
			final Actor v12;
			v12 = f14.m115(v11);
			if (v12 != null) {
				v12.m610(v5);
			}
			v11++;
		}

        // Debug
        if (f620.m53()) {
            v5.setColor(Color.RED);
            v5.drawString("DEBUG ON", 750, 650);
            /*
             * // Paint gridline overlay for(int i = 0; i < mapWidth; i++)
             * g.drawLine(i*map.CELL_SIZE, 0, i*map.CELL_SIZE,
             * mapHeight*map.CELL_SIZE); for(int i = 0; i < mapHeight; i++)
             * g.drawLine(0, i*map.CELL_SIZE, mapWidth*map.CELL_SIZE,
             * i*map.CELL_SIZE);
             */

            // Player X,Y coordinates bottom right
            v5.drawString("positionX: " + f04.m310(), 750, 675);
            v5.drawString("positionY: " + f04.m410(), 750, 700);
        }

        // Check for game pause and print pause status
        if (f74) {
            v5.setColor(Color.RED);
            v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
            v5.drawString("PAUSED", 750, 500);
            if (f84 > System.currentTimeMillis()) {
                v5.drawString(
                        "Pause ends in..." + ((f84 - System.currentTimeMillis()) / 1000),
                        750, 550);
            }
            if (f84 != 0 && System.currentTimeMillis() > f84) {
                f84 = 0;
                f74 = false;
            }
            return;
        }
    }

    /**
     * Player has died or the Supervisor has decided to change the state
     * abruptly
     * 
     * @see State#m320()
     */
    @Override
    public void m320() {
        // Cleanup
        f04 = null;
        f14 = null;
    }

    /**
     * Player has won, move to the next level Called by logic()
     * 
     * @see StateGame#m220()
     */
    public void m24() {
        f54 += f04.m115();

        m14(true);
    }

    /**
     * Player has died, reset() if lives remain. Otherwise, request a state
     * change thereby end()ing this state Called by logic()
     * 
     * @see StateGame#m220()
     */
    public void m34() {
        f64--;

        if (f64 > 0) {
            m14(false);
        } else {
            if (f44 == 1) {
                f54 = f04.m115(); // win() never called, so
                                                  // score is the 1st level
                                                  // score
            }
            f620.m73(State.f120);
        }
    }

    /**
     * Start automove in certain direction
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v13) {
        if (f04 == null) {
            return;
        }

        switch (v13.getKeyCode()) {
            case KeyEvent.VK_UP:
                f04.m69(Direction.f00);
                break;
            case KeyEvent.VK_RIGHT:
                f04.m69(Direction.f10);
                break;
            case KeyEvent.VK_DOWN:
                f04.m69(Direction.f20);
                break;
            case KeyEvent.VK_LEFT:
                f04.m69(Direction.f30);
                break;
            case KeyEvent.VK_SPACE:
                f04.m69(Direction.f50);
                break;
            case KeyEvent.VK_P:
                // Don't interupt system pauses
                if (f84 < System.currentTimeMillis()) {
                    f74 = !f74;
                }
                break;
            case KeyEvent.VK_V:
                f620.m63();
                // AI debug
                f24.m07(f620.m53());
                break;
            case KeyEvent.VK_0:
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }
}
