package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FilenameFilter;

import pacman.game.Game;

/**
 * StateMenu is a graphical representation that allows users to switch into
 * other States of the program like the Game and editor
 * 
 * @author Ramsey Kant
 */
public class StateMenu extends State {

    // Private instance
    private int f02;
    private int f12;
    private byte f22;
    private byte f32; // Corresponds to the index in mapList
    private String[] f42;

    public StateMenu(Game v0) {
        super(v0);
    }

    @Override
    public void m120() {
        // Set cursor & menu position
        f02 = 380;
        f12 = 310;
        f22 = 0;
        f32 = 0;

        final File v1;
		v1 = new File(System.getProperty("user.dir"));

        final FilenameFilter v2;
		v2 = new FilenameFilter() {
			@Override
			public boolean accept(File v3, String v4) {
				return v4.endsWith(".map");
			}
		};

        // Apply the filter
        f42 = v1.list(v2);

        if (f42 == null) {
            System.out.println("No maps exist!");
            f620.m73(f520);
            return;
        }
    }

    /**
     * Cleanup Menu objects
     * 
     * @see State#m320()
     */
    @Override
    public void m320() {
        // do nothing
    }

    /**
     * Logic processing for the Menu. Rendering, Input, screen pointer
     * manipulation
     * 
     * @see State#m220()
     */
    @Override
    public void m220() {
        final Graphics2D v5;
		v5 = f620.m23();

        // Draw title
        v5.setColor(Color.YELLOW);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 50));
        v5.fillArc(56, 92, 100, 100, 35, 270); // First pacman
        v5.drawString("PACMAN", 350, 180);
        v5.fillArc(780, 92, 100, 100, 35, 270);

        // Draw menu options
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("Play Game", 380, 300);
        //g.drawString("Map Editor", 525, 340);
        v5.drawString("Scoreboard", 380, 340);
        v5.drawString("Exit", 380, 380);
        if (f42.length > 0) {
            v5.drawString("Current Map: " + f42[f32], 380, 600);
        } else {
            v5.drawString(
                    "No maps detected. Have you placed the maps file in the same directory as the program?",
                    100, 600);
        }

        // Draw underline cursor
        v5.setColor(Color.RED);
        v5.fillRect(f02, f12, 150, 5);
    }

    @Override
    public void keyPressed(KeyEvent v6) {
        switch (v6.getKeyCode()) {
            case KeyEvent.VK_RIGHT:
                if (f32 >= 0 && f32 < (f42.length - 1)) {
                    f32++;
                }
                break;
            case KeyEvent.VK_LEFT:
                if (f32 > 0 && f32 <= (f42.length - 1)) {
                    f32--;
                }
                break;
            case KeyEvent.VK_DOWN:
                if (f22 >= 0 && f22 < 2) {
                    f22++;
                    f12 += 38;
                }
                break;
            case KeyEvent.VK_UP:
                if (f22 > 0 && f22 <= 2) {
                    f22--;
                    f12 -= 38;
                }
                break;
            case KeyEvent.VK_ENTER:
                // Execute the appropriate state change
                switch (f22) {
                    case 0:
                        // Play game
                        if (f42.length > 0) {
                            f620.m43(f42[f32]);
                            f620.m73(f220);
                        }
                        break;
                    case 1:
                        // Scoreboard
                        f620.m73(f120);
                        break;
                    case 2:
                        // Exit
                        f620.m73(f520);
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }

}
