package pacman.map;
import java.util.ArrayList;

/**
 * A path determined by some path finding algorithm. A series of steps from the
 * starting location to the target location. This includes a step for the
 * initial location.
 * 
 * @author Kevin Glass
 */
public class Path {
    /** The list of steps building up this path */
    private final ArrayList<Step> f03;

    /**
     * Create an empty path
     */
    public Path() {
        f03 = new ArrayList<Step>();
    }

    /**
     * Get the length of the path, i.e. the number of steps
     * 
     * @return The number of steps in this path
     */
    public int m03() {
        return f03.size();
    }

    /**
     * Get the step at a given index in the path
     * 
     * @param v0
     *            The index of the step to retrieve. Note this should be >= 0
     *            and < getLength();
     * @return The step information, the position on the map.
     */
    public Step m13(int v0) {
        return f03.get(v0);
    }

    /**
     * Get the x coordinate for the step at the given index
     * 
     * @param v1
     *            The index of the step whose x coordinate should be retrieved
     * @return The x coordinate at the step
     */
    public int m23(int v1) {
        return m13(v1).f03;
    }

    /**
     * Get the y coordinate for the step at the given index
     * 
     * @param v2
     *            The index of the step whose y coordinate should be retrieved
     * @return The y coordinate at the step
     */
    public int m33(int v2) {
        return m13(v2).f13;
    }

    /**
     * Append a step to the path.
     * 
     * @param v3
     *            The x coordinate of the new step
     * @param v4
     *            The y coordinate of the new step
     */
    public void m43(int v3, int v4) {
        f03.add(new Step(v3, v4));
    }

    /**
     * Prepend a step to the path.
     * 
     * @param v5
     *            The x coordinate of the new step
     * @param v6
     *            The y coordinate of the new step
     */
    public void m53(int v5, int v6) {
        f03.add(0, new Step(v5, v6));
    }

    /**
     * Check if this path contains the given step
     * 
     * @param v7
     *            The x coordinate of the step to check for
     * @param v8
     *            The y coordinate of the step to check for
     * @return True if the path contains the given step
     */
    public boolean m63(int v7, int v8) {
        return f03.contains(new Step(v7, v8));
    }

    /**
     * A single step within the path
     * 
     * @author Kevin Glass
     */
    public class Step {
        /** The x coordinate at the given step */
        private final int f03;
        /** The y coordinate at the given step */
        private final int f13;

        /**
         * Create a new step
         * 
         * @param v9
         *            The x coordinate of the new step
         * @param v10
         *            The y coordinate of the new step
         */
        public Step(int v9, int v10) {
            this.f03 = v9;
            this.f13 = v10;
        }

        /**
         * Get the x coordinate of the new step
         * 
         * @return The x coodindate of the new step
         */
        public int m03() {
            return f03;
        }

        /**
         * Get the y coordinate of the new step
         * 
         * @return The y coodindate of the new step
         */
        public int m13() {
            return f13;
        }

        /**
         * @see Object#hashCode()
         */
        @Override
        public int hashCode() {
            return f03 * f13;
        }

        /**
         * @see Object#equals(Object)
         */
        @Override
        public boolean equals(Object v11) {
            if (v11 instanceof Step) {
                final Step v12;
				v12 = (Step) v11;

                return (v12.f03 == f03) && (v12.f13 == f13);
            }

            return false;
        }
    }
}
