package pacman.state;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.editor.EditorFrame;
import pacman.editor.EditorMarker;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;

/**
 * The StateEditor is a mode of the program that allows the user to create and
 * modify map files that can be played in StateGame. StateEditor is a subclass
 * of State
 * 
 * @author Ramsey Kant
 */
public class StateEditor extends State {

    // Logic object references
    private final EditorFrame f02;
    private EditorMarker f12;
    private boolean f22;
    private Map f32;

    // Placement variables
    private int f42;
    private byte f52;
    private String f62;
    private boolean f72;
    private int f82;
    private int f92;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f102;
    private int f112;

    public StateEditor(Game v0) {
        super(v0);

        // If true, remove all editor helpers like grid lines
        f22 = false;

        // Create the editor toolpane
        f68.m120().setSize(1024, f68.f520);
        f02 = new EditorFrame(this);
        f02.setVisible(true);

        // Defaults
        f42 = GameObject.f616;
        f52 = GameObject.f816;
        f62 = "Blinky";
        f72 = false;
        f82 = 13;
        f92 = 17;
    }

    // Getters and Setters

    /**
     * Set the type of object to be placed by the marker Called by the
     * EditorFrame (Dot, Powerup, Teleport buttons)
     * 
     * @param v1
     *            Type of object (from GameObject statics)
     */
    public void m02(int v1) {
        f42 = v1;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Wall button
     * 
     * @param v2
     *            Type of wall (from GameObject statics)
     */
    public void m12(byte v2) {
        f52 = v2;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Add Ghost button
     * 
     * @param v3
     *            Type of wall (from GameObject statics)
     */
    public void m22(String v3) {
        f62 = v3;
    }

    /**
     * Toggle the trapped status of the next ghost to be added
     * 
     * @param v4
     *            True if trapped in the spawn-jail
     */
    public void m32(boolean v4) {
        f72 = v4;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if new ghosts will be created as trapped
     */
    public boolean m42() {
        return f72;
    }

    /**
     * Set the teleport destination, used to aid a teleport drop Called in
     * EditorFrame by the Teleport add button
     * 
     * @param v5
     *            destination coordinate X of the next teleport
     * @param v6
     *            destination coordinate Y of the next teleport
     */
    public void m52(int v5, int v6) {
        f82 = v5;
        f92 = v6;
    }

    /**
     * Reset the StateEditor objects like the Marker
     * 
     * @see State#m18()
     */
    @Override
    public void m18() {
        // Force previous references out of scope
        f12 = null;
        f32 = null;

        f42 = GameObject.f016;
    }

    /**
     * Setup and render a new blank map
     * 
     * @param v7
     *            The width of the map to be created
     * @param v8
     *            The height of the map to be created
     */
    public void m62(int v7, int v8) {
        // Setup the game map
        f68.m220().setBackground(Color.BLACK);
        f102 = v7;
        f112 = v8;
        f32 = new Map(28, 31, 32);

        // Create the marker (but don't put it "in" the map)
        f12 = new EditorMarker(Color.GREEN, f32, 0, 0);
    }

    /**
     * Save the map
     * 
     * @param v9
     */
    public void m72(String v9) {
        f32.m235(System.getProperty("user.dir") + "\\" + v9);
    }

    /**
     * Setup and render a map loaded from the file system
     * 
     * @param v10
     */
    public void m82(String v10) {
        // Setup the game map
        f68.m220().setBackground(Color.BLACK);
        f32 = new Map(System.getProperty("user.dir") + "\\" + v10, 32);
        f102 = f32.m05();
        f112 = f32.m15();

        // Create the marker (but don't put it "in" the map)
        f12 = new EditorMarker(Color.GREEN, f32, 0, 0);
    }

    /**
     * Logic of the editor processed here: Rendering, input, and object
     * placement. Called in the mainThreadLoop
     * 
     * @see State#m28()
     */
    @Override
    public void m28() {
        if (f32 == null) {
            return;
        }

        final Graphics2D v11;
		v11 = f68.m220();

        // Offset the buffer so object's arent clipped by the window borders
        v11.translate(10, 30);

        Item v12;
		v12 = null;
        int v13;
		for (v13 = 0; v13 < f102; v13 = v13 + 1) {
            int v14;
			for (v14 = 0; v14 < f112; v14 = v14 + 1) {
                final byte v15;
				v15 = f32.m95(v13, v14);

                // Switch based on wall type and paint
                v11.setColor(Color.BLUE);
                switch (v15) {
                    case 0:
                        // Nothing
                        break;
                    case GameObject.f816:
                        // Vertical wall, no edges
                        v11.fillRoundRect(v13 * f32.f25 + 10, v14 * f32.f25, 12,
                                f32.f25, 0, 0); // 2x+12 = map.CELL_SIZE.
                                                      // x = 10
                        break;
                    case GameObject.f916:
                        // Horizontal wall, no edges
                        v11.fillRoundRect(v13 * f32.f25, v14 * f32.f25 + 10, f32.f25,
                                12, 0, 0);
                        break;
                    case GameObject.f1016:
                        // g.fillArc(x*map.CELL_SIZE+10, y*map.CELL_SIZE,
                        // map.CELL_SIZE, map.CELL_SIZE, 90, 90);
                        v11.fillRoundRect(v13 * f32.f25 + (f32.f25 / 2), v14 * f32.f25
                                + 10, f32.f25 / 2, 12, 0, 0);
                        v11.fillRoundRect(v13 * f32.f25 + 10, v14 * f32.f25
                                + (f32.f25 / 2), 12, f32.f25 / 2, 0, 0);
                        break;
                    case GameObject.f1116:
                        v11.fillRoundRect(v13 * f32.f25, v14 * f32.f25 + 10,
                                f32.f25 / 2, 12, 0, 0);
                        v11.fillRoundRect(v13 * f32.f25 + 10, v14 * f32.f25
                                + (f32.f25 / 2), 12, f32.f25 / 2, 0, 0);
                        break;
                    case GameObject.f1216:
                        v11.fillRoundRect(v13 * f32.f25 + (f32.f25 / 2), v14 * f32.f25
                                + 10, f32.f25 / 2, 12, 0, 0); // hori
                        v11.fillRoundRect(v13 * f32.f25 + 10, v14 * f32.f25, 12,
                                f32.f25 / 2, 0, 0); // vert
                        break;
                    case GameObject.f1316:
                        v11.fillRoundRect(v13 * f32.f25, v14 * f32.f25 + 10,
                                f32.f25 / 2, 12, 0, 0); // hori
                        v11.fillRoundRect(v13 * f32.f25 + 10, v14 * f32.f25, 12,
                                f32.f25 / 2, 0, 0); // vert
                        break;
                    case GameObject.f1416:
                        v11.setColor(Color.PINK);
                        v11.fillRoundRect(v13 * f32.f25, v14 * f32.f25 + 10, f32.f25,
                                6, 0, 0);
                        break;
                    default:
                        break;
                }

                // Paint any of the items here
                v12 = f32.m105(v13, v14);
                if (v12 != null) {
                    v12.m616(v11);
                }
            }
        }

        final int v16;
		v16 = f32.m25();
        int v17;
		for (v17 = 0; v17 < v16; v17 = v17 + 1) {
            final Actor v18;
			v18 = f32.m115(v17);
            if (v18 != null) {
                v18.m616(v11);
            }
        }

        // Paint the marker
        f12.m616(v11);

        // Paint gridline overlay if in editor view
        if (!f22) {
            v11.setColor(Color.RED);
            int v19;
			for (v19 = 0; v19 < f102; v19 = v19 + 1) {
                v11.drawLine(v19 * f32.f25, 0, v19 * f32.f25, f112 * f32.f25);
            }
            int v20;
			for (v20 = 0; v20 < f112; v20 = v20 + 1) {
                v11.drawLine(0, v20 * f32.f25, f102 * f32.f25, v20 * f32.f25);
            }

            // Player X,Y coordinates bottom right
            v11.drawString("X: " + f12.m316() + ", Y: " + f12.m416(), 900, 700);
        }
    }

    /**
     * Termination of the StateEdtior. Set references stored by the StateEditor
     * as null
     * 
     * @see State#m38()
     */
    @Override
    public void m38() {
        // Cleanup
        f12 = null;
        f32 = null;
    }

    /**
     * Input processing for the Editor
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v21) {
        switch (v21.getKeyCode()) {
            case KeyEvent.VK_UP:
                f12.m09(0, -1);
                break;
            case KeyEvent.VK_RIGHT:
                f12.m09(1, 0);
                break;
            case KeyEvent.VK_DOWN:
                f12.m09(0, +1);
                break;
            case KeyEvent.VK_LEFT:
                f12.m09(-1, 0);
                break;
            case KeyEvent.VK_ENTER:
                if (f12 == null) {
                    return;
                }

                // If not empty, bail
                if (!f32.m185(f12.m316(), f12.m416())) {
                    return;
                }

                switch (f42) {
                    case GameObject.f616:
                        f32.m65(f12.m316(), f12.m416(), f52);
                        break;
                    case GameObject.f016:
                        f32.m75(new Item(GameObject.f016, Color.WHITE, f32,
                                f12.m316(), f12.m416()));
                        break;
                    case GameObject.f116:
                        f32.m75(new Item(GameObject.f116, Color.WHITE, f32, f12
                                .m316(), f12.m416()));
                        break;
                    case GameObject.f416:
                        if (f62.equals("Blinky")) {
                            f32.m85(new Ghost(Color.RED, f32, f12.m316(), f12.m416(),
                                    f72));
                        } else if (f62.equals("Pinky")) {
                            f32.m85(new Ghost(Color.PINK, f32, f12.m316(), f12.m416(),
                                    f72));
                        } else if (f62.equals("Inky")) {
                            f32.m85(new Ghost(Color.CYAN, f32, f12.m316(), f12.m416(),
                                    f72));
                        } else {
                            f32.m85(new Ghost(Color.ORANGE, f32, f12.m316(), f12.m416(),
                                    f72));
                        }
                        break;
                    case GameObject.f316:
					int v22;
					// If there is already a player in the actors list,
                        // remove it
                        for (v22 = 0; v22 < f32.m25(); v22++) {
                            if (f32.m115(v22).m016() == GameObject.f316) {
                                f32.m145(v22);
                                v22 = v22 - 1;
                            }
                        }

                        // Add the new player
                        f32.m85(new Player(f32, f12.m316(), f12.m416()));
                        break;
                    case GameObject.f716:
					final Item v23;
					v23 = new Item(GameObject.f716,
							Color.LIGHT_GRAY, f32, f12.m316(), f12.m416());
                        v23.m07(f82, f92);
                        f32.m75(v23);
                        break;

                    default:
                        break;
                }
                break;
            case KeyEvent.VK_DELETE:
                // Delete a placed object. Will reduce excessive memory
                // consumption if the user cant just replace a tile with a new
                // object

                // If empty, bail
                if (f32.m185(f12.m316(), f12.m416())) {
                    return;
                }

                // Remove anything (collidable, actor, or item) at (x,y)
                f32.m165(f12.m316(), f12.m416());
                break;
            case KeyEvent.VK_V:
                f22 = !f22;
                break;
            case KeyEvent.VK_0:
                // editorFrame.setEnabled(false);
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }

}
