package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FilenameFilter;

import pacman.game.Game;

/**
 * StateMenu is a graphical representation that allows users to switch into
 * other States of the program like the Game and editor
 * 
 * @author Ramsey Kant
 */
public class StateMenu extends State {

    // Private instance
    private int f019;
    private int f119;
    private byte f219;
    private byte f319; // Corresponds to the index in mapList
    private String[] f419;

    public StateMenu(Game v0) {
        super(v0);
    }

    @Override
    public void m18() {
        // Set cursor & menu position
        f019 = 380;
        f119 = 310;
        f219 = 0;
        f319 = 0;

        final File v1;
		v1 = new File(System.getProperty("user.dir"));

        final FilenameFilter v2;
		v2 = new FilenameFilter() {
			@Override
			public boolean accept(File v3, String v4) {
				return v4.endsWith(".map");
			}
		};

        // Apply the filter
        f419 = v1.list(v2);

        if (f419 == null) {
            System.out.println("No maps exist!");
            f68.m720(f58);
            return;
        }
    }

    /**
     * Cleanup Menu objects
     * 
     * @see State#m38()
     */
    @Override
    public void m38() {
        // do nothing
    }

    /**
     * Logic processing for the Menu. Rendering, Input, screen pointer
     * manipulation
     * 
     * @see State#m28()
     */
    @Override
    public void m28() {
        final Graphics2D v5;
		v5 = f68.m220();

        // Draw title
        v5.setColor(Color.YELLOW);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 50));
        v5.fillArc(56, 92, 100, 100, 35, 270); // First pacman
        v5.drawString("PACMAN", 350, 180);
        v5.fillArc(780, 92, 100, 100, 35, 270);

        // Draw menu options
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("Play Game", 380, 300);
        //g.drawString("Map Editor", 525, 340);
        v5.drawString("Scoreboard", 380, 340);
        v5.drawString("Exit", 380, 380);
        if (f419.length > 0) {
            v5.drawString("Current Map: " + f419[f319], 380, 600);
        } else {
            v5.drawString(
                    "No maps detected. Have you placed the maps file in the same directory as the program?",
                    100, 600);
        }

        // Draw underline cursor
        v5.setColor(Color.RED);
        v5.fillRect(f019, f119, 150, 5);
    }

    @Override
    public void keyPressed(KeyEvent v6) {
        switch (v6.getKeyCode()) {
            case KeyEvent.VK_RIGHT:
                if (f319 >= 0 && f319 < (f419.length - 1)) {
                    f319 = (byte) (f319 + 1);
                }
                break;
            case KeyEvent.VK_LEFT:
                if (f319 > 0 && f319 <= (f419.length - 1)) {
                    f319 = (byte) (f319 - 1);
                }
                break;
            case KeyEvent.VK_DOWN:
                if (f219 >= 0 && f219 < 2) {
                    f219 = (byte) (f219 + 1);
                    f119 = f119 + (38);
                }
                break;
            case KeyEvent.VK_UP:
                if (f219 > 0 && f219 <= 2) {
                    f219 = (byte) (f219 - 1);
                    f119 = f119 - (38);
                }
                break;
            case KeyEvent.VK_ENTER:
                // Execute the appropriate state change
                switch (f219) {
                    case 0:
                        // Play game
                        if (f419.length > 0) {
                            f68.m420(f419[f319]);
                            f68.m720(f28);
                        }
                        break;
                    case 1:
                        // Scoreboard
                        f68.m720(f18);
                        break;
                    case 2:
                        // Exit
                        f68.m720(f58);
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }

}
