package pacman.actors;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.ai.AIManager;
import pacman.game.GameObject;
import pacman.map.Map;
import pacman.map.Path;
import pacman.util.Direction;

/**
 * The Ghost class is the primary enemy in Pacman. Intelligent decisions of
 * ghosts are made by the AIManager class Ghost is a subclass of Actor
 * 
 * @author Ramsey Kant
 */
public class Ghost extends Actor {
    // Movement
    private Path f04;
    private int f14;
    private boolean f24;

    // State
    private boolean f34;
    private boolean f44;
    private boolean f54;

    /**
     * Class constructor
     * 
     * @param v0
     *            Color of the ghost's 'body'
     * @param v1
     *            Reference to the map
     * @param v2
     *            X coordinate to spawn at
     * @param v3
     *            Y coordinate to spawn at
     * @param v4
     *            Set trapped status
     */
    public Ghost(Color v0, Map v1, int v2, int v3, boolean v4) {
        super(GameObject.f420, v0, v1, v2, v3);
        f24 = true;
        f44 = false;
        f34 = v4;
        f54 = false;
    }

    /**
     * Return the fear status of the ghost
     * 
     * @return True if fearful
     */
    public boolean m04() {
        return f44;
    }

    /**
     * Set fear status. AIManager interperates this setting for behavior
     * 
     * @param v5
     *            Fear status, true if fearful
     */
    public void m14(boolean v5) {
        f44 = v5;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if the ghost is currently in the spawn-jail
     */
    public boolean m24() {
        return f34;
    }

    /**
     * Set the current trapped status
     * 
     * @param v6
     *            Trye uf the ghost is in the spawn-jail
     */
    public void m34(boolean v6) {
        f34 = v6;
    }

    /**
     * Flag that is set to true when the path reaches the last possible step
     * 
     * @return True if the AIManager needs to assign a new path
     */
    public boolean m44() {
        return f24;
    }

    /**
     * Update the Path object for the ghost to follow'
     * 
     * @param v7
     *            Path object generated in process() by the AIManager
     * @see AIManager#m20()
     */
    public void m54(Path v7) {
        f14 = 1;
        f04 = v7;
        f24 = false;
    }

    /**
     * Direct's the paint() function to draw the current path of the ghost on
     * the map
     * 
     * @param v8
     *            If true, debug is on and the path will be drawn
     * @see AIManager#setDebugEnabled
     */
    public void m64(boolean v8) {
        f54 = v8;
    }

    /**
     * Run a think cycle for the AI. Major decisions are made by the AIManager
     * (pathing), this just determines movement and screen draw deltas
     * 
     * @see Actor#m520()
     */
    @Override
    public void m520() {
        // Move to the next step
        if (f04 != null && f14 < f04.m010()) {

            // Figure out the direction
            if ((f04.m310(f14) - f1820) < 0) {
                f36 = Direction.f07;
            } else if ((f04.m310(f14) - f1820) > 0) {
                f36 = Direction.f27;
            } else if ((f04.m210(f14) - f1720) > 0) {
                f36 = Direction.f17;
            } else {
                f36 = Direction.f37;
            }

            // Based on the direction, move the screen delta's and the X,Y
            // coordinates if the # of pixels for the cell have been surpassed
            switch (f36) {
                case f07:
                    f76 = 0;
                    f86 -= 0 - f96;
                    // If the movement delta has surpassed the number of pixels for
                    // the cell, set him to the map cell he has reached by his movement.
                    if (Math.abs(f86) >= f1920.f29) {
                        f86 = 0;
                        m76(f1720, f1820 - 1);
                        f14 = f14 + 1;
                    }
                    break;
                case f17:
                    f76 += 0 + f96;
                    f86 = 0;
                    if (Math.abs(f76) >= f1920.f29) {
                        f76 = 0;
                        m76(f1720 + 1, f1820);
                        f14 = f14 + 1;
                    }
                    break;
                case f27:
                    f76 = 0;
                    f86 += 0 + f96;
                    if (Math.abs(f86) >= f1920.f29) {
                        f86 = 0;
                        m76(f1720, f1820 + 1);
                        f14 = f14 + 1;
                    }
                    break;
                case f37:
                    f76 -= 0 - f96;
                    f86 = 0;
                    if (Math.abs(f76) >= f1920.f29) {
                        f76 = 0;
                        m76(f1720 - 1, f1820);
                        f14 = f14 + 1;
                    }
                    break;
                case f57:
                case f47:
                    // do not move
            }
        } else {
            f24 = true;
        }
    }

    /**
     * Draw the ghost
     * 
     * @see GameObject#m620(Graphics2D)
     */
    @Override
    public void m620(Graphics2D v9) {
        final int v10;
		v10 = (int) ((f1920.f29 * f1720) + f76);
        final int v11;
		v11 = (int) ((f1920.f29 * f1820) + f86);

        v9.setColor(f1620);

        // Body
        if (f44) {
            v9.setColor(Color.WHITE);
        }
        v9.fillArc(v10, v11, f1920.f29, f1920.f29, 0, 360);
        v9.fillRect((int) ((f1920.f29 * f1720) + f76), (int) ((f1920.f29 * f1820)
                + (f1920.f29 / 2) + f86), f1920.f29, f1920.f29 / 2);

        // Eyes
        if (f44) {
            v9.setColor(Color.BLACK);
        } else {
            v9.setColor(Color.WHITE);
        }
        v9.fillOval((int) ((f1920.f29 * f1720) + 4 + f76),
                (int) ((f1920.f29 * f1820) + 3 + f86), 8, 10);
        v9.fillOval((int) ((f1920.f29 * f1720) + 12 + f76),
                (int) ((f1920.f29 * f1820) + 3 + f86), 8, 10);

        // Eyeballs
        v9.setColor(Color.BLUE);
        v9.fillOval((int) ((f1920.f29 * f1720) + 7 + f76),
                (int) ((f1920.f29 * f1820) + 6 + f86), 4, 4);
        v9.fillOval((int) ((f1920.f29 * f1720) + 13 + f76),
                (int) ((f1920.f29 * f1820) + 6 + f86), 4, 4);

        // Debug draw path
        if (f54 && f04 != null) {
            int v12;
			v12 = 0;
			while (v12 < f04.m010()) {
				final Path.Step v13;
				v13 = f04.m110(v12);
				v9.setColor(f1620);
				v9.drawLine(f1920.f29 * v13.m010(), f1920.f29 * v13.m110(),
						(f1920.f29 * v13.m010()) + f1920.f29,
						(f1920.f29 * v13.m110()) + f1920.f29);
				v12 = v12 + 1;
			}
        }
    }
}
