package pacman.game;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics2D;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;

import pacman.state.State;
import pacman.state.StateEditor;
import pacman.state.StateGame;
import pacman.state.StateMenu;
import pacman.state.StateScoreboard;

/**
 * The Game Supervisor. This class implements that core program logic, state
 * management, and graphics.
 * 
 * @author Ramsey Kant
 */
public class Game extends Canvas {
    private static final long f013 = 1L;

    // Debug vars
    private boolean f113;

    // Threading
    private boolean f213;

    // Graphics variables
    private Frame f313;
    public final int f413;
    public final int f513;
    private BufferStrategy f613;

    // State
    private int f713;
    private State f813;
    private boolean f913;
    private int f1013;
    private String f1113;

    /**
     * Class Constructor Set's up graphics and put's game logic into a startup
     * state by calling init()
     * 
     * @param v0
     *            Resolution X
     * @param v1
     *            Resolution Y
     * @see Game#m013()
     */
    public Game(int v0, int v1) {
        // Set resolution settings
        f413 = v0;
        f513 = v1;

        // Init game
        m013();
    }

    /**
     * Startup functionality for the program called by the constructor
     */
    private void m013() {
        // Debug vars
        f113 = false;

        f1113 = "test.map";
        f913 = false;

        // Setup the game frame
        f313 = new Frame("Pacman");
        f313.setLayout(null);
        setBounds(0, 0, f413, f513);
        f313.add(this);
        f313.setSize(f413, f513);
        f313.setResizable(false);
        f313.setVisible(true);

        // Set the exit handler with an anonymous class
        f313.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent v2) {
                // Exit main thread
                f213 = false;
            }
        });

        // Setup double buffering
        setIgnoreRepaint(true); // We'll handle repainting
        createBufferStrategy(2);
        f613 = getBufferStrategy();

        f213 = true;
    }

    // Getter and Setter methods

    /**
     * Get the Frame object encapsulating the program
     * 
     * @return The frame
     */
    public Frame m113() {
        return f313;
    }

    /**
     * Get a 'handle' of the current graphics buffer for drawing
     * 
     * @return The Graphics2D buffer
     */
    public Graphics2D m213() {
        return (Graphics2D) f613.getDrawGraphics();
    }

    /**
     * Get the name of the map to be loaded in StateGame
     * 
     * @return Map name (with .map extension)
     */
    public String m313() {
        return f1113;
    }

    /**
     * Set the default starting map (set by menu)
     * 
     * @param v3
     *            The name of the map to load (with the .map extension)
     */
    public void m413(String v3) {
        f1113 = v3;
    }

    /**
     * Return the current debug setting
     * 
     * @return True if debug setting is on
     * @see Game#m613()
     */
    public boolean m513() {
        return f113;
    }

    /**
     * Toggle debugging. Facilities like AIManager use this flag to display
     * diagnostic information like AI paths
     */
    public void m613() {
        f113 = !f113;
    }

    // Public Methods

    /**
     * Called by other states to safely change currentState. This is done so the
     * currentState's logic can finish
     * 
     * @see Game#m813()
     */
    public void m713(int v4) {
        f1013 = v4;
        f913 = true;
    }

    /**
     * The main game loop that handles graphics and game state determination
     */
    public void m813() {
    	final long v5;
		v5 = 20;
    	long v6;
		v6 = 0;
    	
        for (; f213;) {
			if ((v6 + v5) > System.currentTimeMillis()) {
				continue;
			}
			v6 = System.currentTimeMillis();
			if (f913) {
				f913 = false;
				m913(f1013);
				continue;
			}
			final Graphics2D v7;
			v7 = m213();
			v7.setColor(Color.black);
			v7.fillRect(0, 0, f413, f513);
			f813.m23();
			v7.dispose();
			f613.show();
			try {
				Thread.sleep(10);
			} catch (InterruptedException v8) {
				v8.printStackTrace();
			}
		}
    }

    // Private Methods

    /**
     * Change the state of the game. Called in mainThreadLogic()
     * 
     * @param v9
     *            The state to set. Must match the static vars above
     * @see Game#m713(int)
     * @see Game#m813()
     */
    private void m913(int v9) {
        // Cleanup for the outgoing state
        if (f813 != null) {
            f313.removeKeyListener(f813);
            removeKeyListener(f813);
            f813.m33();
        }

        // Set the new state type
        f713 = v9;

        // Instance the new state (reset() is called in the construtor)
        switch (f713) {
            case State.f23:
                f813 = new StateGame(this);
                break;
            case State.f13:
                f813 = new StateScoreboard(this);
                /*
                 * StateGame sb = new StateScoreboard(); int newScore = 0;
                 * 
                 * // If the previous state was STATE_GAME, pull the session
                 * score and pass it to the scoreboard if(currentState
                 * instanceof StateGame)
                 * sb.addScore((int)((StateGame)currentState
                 * ).getSessionScore()));
                 * 
                 * currentState = sb;
                 */
                break;
            case State.f43:
                f813 = new StateEditor(this);
                break;
            case State.f03:
                f813 = new StateMenu(this);
                break;
            case State.f53:
                f813 = null;
                f213 = false;
                break;
            default:
                break;
        }

        // Setup input handler and reset()
        if (f813 != null) {
            f313.addKeyListener(f813);
            addKeyListener(f813);
        }
    }
}
