package pacman.map;
import java.util.ArrayList;
import java.util.Collections;

import pacman.actors.Actor;
import pacman.ai.AStarHeuristic;

/**
 * A path finder implementation that uses the AStar heuristic based algorithm to
 * determine a path.
 * 
 * @author Kevin Glass
 */
public class PathFinder {
    /** The set of nodes that have been searched through */
    private final ArrayList<Node> f05;
    /** The set of nodes that we do not yet consider fully searched */
    private final SortedNodeList f15 = new SortedNodeList();

    /** The map being searched */
    private final Map f25;
    /** The maximum depth of search we're willing to accept before giving up */
    private final int f35;

    /** The complete set of nodes across the map */
    private final Node[][] f45;
    /** True if we allow diaganol movement */
    private final boolean f55;
    /** The heuristic we're applying to determine which nodes to search first */
    private final AStarHeuristic f65;

    /**
     * Create a path finder with the default heuristic - closest to target.
     * 
     * @param v0
     *            The map to be searched
     * @param v1
     *            The maximum depth we'll search before giving up
     * @param v2
     *            True if the search should try diaganol movement
     */
    public PathFinder(Map v0, int v1, boolean v2) {
        this(v0, v1, v2, new AStarHeuristic());
    }

    /**
     * Create a path finder
     * 
     * @param v6
     *            The heuristic used to determine the search order of the map
     * @param v3
     *            The map to be searched
     * @param v4
     *            The maximum depth we'll search before giving up
     * @param v5
     *            True if the search should try diaganol movement
     */
    public PathFinder(Map v3, int v4, boolean v5,
            AStarHeuristic v6) {
        this.f65 = v6;
        this.f25 = v3;
        this.f35 = v4;
        this.f55 = v5;

        f05 = new ArrayList<Node>();
        f45 = new Node[v3.m09()][v3.m19()];

        int v7;
		v7 = 0;
		while (v7 < v3.m09()) {
			int v8;
			v8 = 0;
			while (v8 < v3.m19()) {
				f45[v7][v8] = new Node(v7, v8);
				v8 = v8 + 1;
			}
			v7 = v7 + 1;
		}
    }

    /**
     * Find a path from the starting location provided (sx,sy) to the target
     * location (tx,ty) avoiding blockages and attempting to honour costs
     * provided by the tile map.
     * 
     * @param v9
     *            The entity that will be moving along the path. This provides a
     *            place to pass context information about the game entity doing
     *            the moving, e.g. can it fly? can it swim etc.
     * 
     * @param v10
     *            The x coordinate of the start location
     * @param v11
     *            The y coordinate of the start location
     * @param v12
     *            The x coordinate of the target location
     * @param v13
     *            Teh y coordinate of the target location
     * @return The path found from start to end, or null if no path can be
     *         found.
     */

    public Path m05(Actor v9, int v10, int v11, int v12, int v13) {
        // easy first check, if the destination is blocked, we can't get there

        if (!f25.m199(v9, v12, v13)) {
            return null;
        }

        // initial state for A*. The closed group is empty. Only the starting

        // tile is in the open list and it'e're already there
        f45[v10][v11].f25 = 0;
        f45[v10][v11].f55 = 0;
        f05.clear();
        f15.m15();
        f15.m25(f45[v10][v11]);

        f45[v12][v13].f35 = null;

        int v14;
		v14 = 0;
        for (; (v14 < f35) && (f15.m45() != 0);) {
			final Node v15;
			v15 = m15();
			if (v15 == f45[v12][v13]) {
				break;
			}
			m45(v15);
			m55(v15);
			int v16;
			v16 = -1;
			while (v16 < 2) {
				int v17;
				v17 = -1;
				while (v17 < 2) {
					if ((v16 == 0) && (v17 == 0)) {
						continue;
					}
					if (!f55) {
						if ((v16 != 0) && (v17 != 0)) {
							continue;
						}
					}
					final int v18;
					v18 = v16 + v15.f05;
					final int v19;
					v19 = v17 + v15.f15;
					if (m85(v9, v10, v11, v18, v19)) {
						final float v20;
						v20 = v15.f25 + m95(v9, v15.f05, v15.f15, v18, v19);
						final Node v21;
						v21 = f45[v18][v19];
						if (v20 < v21.f25) {
							if (m35(v21)) {
								m45(v21);
							}
							if (m65(v21)) {
								m75(v21);
							}
						}
						if (!m35(v21) && !(m65(v21))) {
							v21.f25 = v20;
							v21.f45 = m105(v9, v18, v19, v12, v13);
							v14 = Math.max(v14, v21.m05(v15));
							m25(v21);
						}
					}
					v17 = v17 + 1;
				}
				v16 = v16 + 1;
			}
		}

        // since we'e've run out of search
        // there was no path. Just return null

        if (f45[v12][v13].f35 == null) {
            return null;
        }

        // At this point we've definitely found a path so we can uses the parent

        // references of the nodes to find out way from the target location back

        // to the start recording the nodes on the way.

        final Path v22;
		v22 = new Path();
        Node v23;
		v23 = f45[v12][v13];
        for (; v23 != f45[v10][v11];) {
			v22.m510(v23.f05, v23.f15);
			v23 = v23.f35;
		}
        v22.m510(v10, v11);

        // thats it, we have our path

        return v22;
    }

    /**
     * Get the first element from the open list. This is the next one to be
     * searched.
     * 
     * @return The first element in the open list
     */
    protected Node m15() {
        return (Node) f15.m05();
    }

    /**
     * Add a node to the open list
     * 
     * @param v24
     *            The node to be added to the open list
     */
    protected void m25(Node v24) {
        f15.m25(v24);
    }

    /**
     * Check if a node is in the open list
     * 
     * @param v25
     *            The node to check for
     * @return True if the node given is in the open list
     */
    protected boolean m35(Node v25) {
        return f15.m55(v25);
    }

    /**
     * Remove a node from the open list
     * 
     * @param v26
     *            The node to remove from the open list
     */
    protected void m45(Node v26) {
        f15.m35(v26);
    }

    /**
     * Add a node to the closed list
     * 
     * @param v27
     *            The node to add to the closed list
     */
    protected void m55(Node v27) {
        f05.add(v27);
    }

    /**
     * Check if the node supplied is in the closed list
     * 
     * @param v28
     *            The node to search for
     * @return True if the node specified is in the closed list
     */
    protected boolean m65(Node v28) {
        return f05.contains(v28);
    }

    /**
     * Remove a node from the closed list
     * 
     * @param v29
     *            The node to remove from the closed list
     */
    protected void m75(Node v29) {
        f05.remove(v29);
    }

    /**
     * Check if a given location is valid for the supplied mover
     * 
     * @param v30
     *            The mover that would hold a given location
     * @param v31
     *            The starting x coordinate
     * @param v32
     *            The starting y coordinate
     * @param v33
     *            The x coordinate of the location to check
     * @param v34
     *            The y coordinate of the location to check
     * @return True if the location is valid for the given mover
     */
    protected boolean m85(Actor v30, int v31, int v32, int v33, int v34) {
        boolean v35;
		v35 = (v33 < 0) || (v34 < 0) || (v33 >= f25.m09())
				|| (v34 >= f25.m19());

        if ((!v35) && ((v31 != v33) || (v32 != v34))) {
            v35 = f25.m199(v30, v33, v34) == false;
        }

        return !v35;
    }

    /**
     * Get the cost to move through a given location
     * 
     * @param v36
     *            The entity that is being moved
     * @param v37
     *            The x coordinate of the tile whose cost is being determined
     * @param v38
     *            The y coordiante of the tile whose cost is being determined
     * @param v39
     *            The x coordinate of the target location
     * @param v40
     *            The y coordinate of the target location
     * @return The cost of movement through the given tile
     */
    public float m95(Actor v36, int v37, int v38, int v39, int v40) {
        return f25.m229(v36, v37, v38, v39, v40);
    }

    /**
     * Get the heuristic cost for the given location. This determines in which
     * order the locations are processed.
     * 
     * @param v41
     *            The entity that is being moved
     * @param v42
     *            The x coordinate of the tile whose cost is being determined
     * @param v43
     *            The y coordiante of the tile whose cost is being determined
     * @param v44
     *            The x coordinate of the target location
     * @param v45
     *            The y coordinate of the target location
     * @return The heuristic cost assigned to the tile
     */
    public float m105(Actor v41, int v42, int v43, int v44, int v45) {
        return f65.m017(f25, v41, v42, v43, v44, v45);
    }

    /**
     * A simple sorted list
     * 
     * @author kevin
     */
    private class SortedNodeList {
        /** The list of elements */
        private final ArrayList<Node> f05 = new ArrayList<Node>();

        /**
         * Retrieve the first element from the list
         * 
         * @return The first element from the list
         */
        public Object m05() {
            return f05.get(0);
        }

        /**
         * Empty the list
         */
        public void m15() {
            f05.clear();
        }

        /**
         * Add an element to the list - causes sorting
         * 
         * @param v46
         *            The element to add
         */
        public void m25(Node v46) {
            f05.add(v46);
            Collections.sort(f05);
        }

        /**
         * Remove an element from the list
         * 
         * @param v47
         *            The element to remove
         */
        public void m35(Object v47) {
            f05.remove(v47);
        }

        /**
         * Get the number of elements in the list
         * 
         * @return The number of element in the list
         */
        public int m45() {
            return f05.size();
        }

        /**
         * Check if an element is in the list
         * 
         * @param v48
         *            The element to search for
         * @return True if the element is in the list
         */
        public boolean m55(Object v48) {
            return f05.contains(v48);
        }
    }

    /**
     * A single node in the search graph
     */
    private class Node implements Comparable<Object> {
        /** The x coordinate of the node */
        private final int f05;
        /** The y coordinate of the node */
        private final int f15;
        /** The path cost for this node */
        private float f25;
        /** The parent of this node, how we reached it in the search */
        private Node f35;
        /** The heuristic cost of this node */
        private float f45;
        /** The search depth of this node */
        private int f55;

        /**
         * Create a new node
         * 
         * @param v49
         *            The x coordinate of the node
         * @param v50
         *            The y coordinate of the node
         */
        public Node(int v49, int v50) {
            this.f05 = v49;
            this.f15 = v50;
        }

        /**
         * Set the parent of this node
         * 
         * @param v51
         *            The parent node which lead us to this node
         * @return The depth we have no reached in searching
         */
        public int m05(Node v51) {
            f55 = v51.f55 + 1;
            this.f35 = v51;

            return f55;
        }

        /**
         * @see Comparable#compareTo(Object)
         */
        @Override
        public int compareTo(Object v52) {
            final Node v53;
			v53 = (Node) v52;

            final float v54;
			v54 = f45 + f25;
            final float v55;
			v55 = v53.f45 + v53.f25;

            if (v54 < v55) {
                return -1;
            } else if (v54 > v55) {
                return 1;
            } else {
                return 0;
            }
        }
    }
}
