package pacman.state;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.editor.EditorFrame;
import pacman.editor.EditorMarker;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;

/**
 * The StateEditor is a mode of the program that allows the user to create and
 * modify map files that can be played in StateGame. StateEditor is a subclass
 * of State
 * 
 * @author Ramsey Kant
 */
public class StateEditor extends State {

    // Logic object references
    private final EditorFrame f018;
    private EditorMarker f118;
    private boolean f218;
    private Map f318;

    // Placement variables
    private int f418;
    private byte f518;
    private String f618;
    private boolean f718;
    private int f818;
    private int f918;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f1018;
    private int f1118;

    public StateEditor(Game v0) {
        super(v0);

        // If true, remove all editor helpers like grid lines
        f218 = false;

        // Create the editor toolpane
        f63.m113().setSize(1024, f63.f513);
        f018 = new EditorFrame(this);
        f018.setVisible(true);

        // Defaults
        f418 = GameObject.f620;
        f518 = GameObject.f820;
        f618 = "Blinky";
        f718 = false;
        f818 = 13;
        f918 = 17;
    }

    // Getters and Setters

    /**
     * Set the type of object to be placed by the marker Called by the
     * EditorFrame (Dot, Powerup, Teleport buttons)
     * 
     * @param v1
     *            Type of object (from GameObject statics)
     */
    public void m018(int v1) {
        f418 = v1;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Wall button
     * 
     * @param v2
     *            Type of wall (from GameObject statics)
     */
    public void m118(byte v2) {
        f518 = v2;
    }

    /**
     * Set the type of wall to be placed by the marker Called by the EditorFrame
     * Add Ghost button
     * 
     * @param v3
     *            Type of wall (from GameObject statics)
     */
    public void m218(String v3) {
        f618 = v3;
    }

    /**
     * Toggle the trapped status of the next ghost to be added
     * 
     * @param v4
     *            True if trapped in the spawn-jail
     */
    public void m318(boolean v4) {
        f718 = v4;
    }

    /**
     * Get the current trapped status
     * 
     * @return True if new ghosts will be created as trapped
     */
    public boolean m418() {
        return f718;
    }

    /**
     * Set the teleport destination, used to aid a teleport drop Called in
     * EditorFrame by the Teleport add button
     * 
     * @param v5
     *            destination coordinate X of the next teleport
     * @param v6
     *            destination coordinate Y of the next teleport
     */
    public void m518(int v5, int v6) {
        f818 = v5;
        f918 = v6;
    }

    /**
     * Reset the StateEditor objects like the Marker
     * 
     * @see State#m13()
     */
    @Override
    public void m13() {
        // Force previous references out of scope
        f118 = null;
        f318 = null;

        f418 = GameObject.f020;
    }

    /**
     * Setup and render a new blank map
     * 
     * @param v7
     *            The width of the map to be created
     * @param v8
     *            The height of the map to be created
     */
    public void m618(int v7, int v8) {
        // Setup the game map
        f63.m213().setBackground(Color.BLACK);
        f1018 = v7;
        f1118 = v8;
        f318 = new Map(28, 31, 32);

        // Create the marker (but don't put it "in" the map)
        f118 = new EditorMarker(Color.GREEN, f318, 0, 0);
    }

    /**
     * Save the map
     * 
     * @param v9
     */
    public void m718(String v9) {
        f318.m239(System.getProperty("user.dir") + "\\" + v9);
    }

    /**
     * Setup and render a map loaded from the file system
     * 
     * @param v10
     */
    public void m818(String v10) {
        // Setup the game map
        f63.m213().setBackground(Color.BLACK);
        f318 = new Map(System.getProperty("user.dir") + "\\" + v10, 32);
        f1018 = f318.m09();
        f1118 = f318.m19();

        // Create the marker (but don't put it "in" the map)
        f118 = new EditorMarker(Color.GREEN, f318, 0, 0);
    }

    /**
     * Logic of the editor processed here: Rendering, input, and object
     * placement. Called in the mainThreadLoop
     * 
     * @see State#m23()
     */
    @Override
    public void m23() {
        if (f318 == null) {
            return;
        }

        final Graphics2D v11;
		v11 = f63.m213();

        // Offset the buffer so object's arent clipped by the window borders
        v11.translate(10, 30);

        Item v12;
		v12 = null;
        int v13;
		v13 = 0;
		while (v13 < f1018) {
			int v14;
			v14 = 0;
			while (v14 < f1118) {
				final byte v15;
				v15 = f318.m99(v13, v14);
				v11.setColor(Color.BLUE);
				switch (v15) {
				case 0:
					break;
				case GameObject.f820:
					v11.fillRoundRect(v13 * f318.f29 + 10, v14 * f318.f29, 12,
							f318.f29, 0, 0);
					break;
				case GameObject.f920:
					v11.fillRoundRect(v13 * f318.f29, v14 * f318.f29 + 10,
							f318.f29, 12, 0, 0);
					break;
				case GameObject.f1020:
					v11.fillRoundRect(v13 * f318.f29 + (f318.f29 / 2), v14
							* f318.f29 + 10, f318.f29 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f318.f29 + 10, v14 * f318.f29
							+ (f318.f29 / 2), 12, f318.f29 / 2, 0, 0);
					break;
				case GameObject.f1120:
					v11.fillRoundRect(v13 * f318.f29, v14 * f318.f29 + 10,
							f318.f29 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f318.f29 + 10, v14 * f318.f29
							+ (f318.f29 / 2), 12, f318.f29 / 2, 0, 0);
					break;
				case GameObject.f1220:
					v11.fillRoundRect(v13 * f318.f29 + (f318.f29 / 2), v14
							* f318.f29 + 10, f318.f29 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f318.f29 + 10, v14 * f318.f29, 12,
							f318.f29 / 2, 0, 0);
					break;
				case GameObject.f1320:
					v11.fillRoundRect(v13 * f318.f29, v14 * f318.f29 + 10,
							f318.f29 / 2, 12, 0, 0);
					v11.fillRoundRect(v13 * f318.f29 + 10, v14 * f318.f29, 12,
							f318.f29 / 2, 0, 0);
					break;
				case GameObject.f1420:
					v11.setColor(Color.PINK);
					v11.fillRoundRect(v13 * f318.f29, v14 * f318.f29 + 10,
							f318.f29, 6, 0, 0);
					break;
				default:
					break;
				}
				v12 = f318.m109(v13, v14);
				if (v12 != null) {
					v12.m620(v11);
				}
				v14 = v14 + 1;
			}
			v13 = v13 + 1;
		}

        final int v16;
		v16 = f318.m29();
        int v17;
		v17 = 0;
		while (v17 < v16) {
			final Actor v18;
			v18 = f318.m119(v17);
			if (v18 != null) {
				v18.m620(v11);
			}
			v17 = v17 + 1;
		}

        // Paint the marker
        f118.m620(v11);

        // Paint gridline overlay if in editor view
        if (!f218) {
            v11.setColor(Color.RED);
            int v19;
			v19 = 0;
			while (v19 < f1018) {
				v11.drawLine(v19 * f318.f29, 0, v19 * f318.f29, f1118
						* f318.f29);
				v19 = v19 + 1;
			}
            int v20;
			v20 = 0;
			while (v20 < f1118) {
				v11.drawLine(0, v20 * f318.f29, f1018 * f318.f29, v20
						* f318.f29);
				v20 = v20 + 1;
			}

            // Player X,Y coordinates bottom right
            v11.drawString("X: " + f118.m320() + ", Y: " + f118.m420(), 900, 700);
        }
    }

    /**
     * Termination of the StateEdtior. Set references stored by the StateEditor
     * as null
     * 
     * @see State#m33()
     */
    @Override
    public void m33() {
        // Cleanup
        f118 = null;
        f318 = null;
    }

    /**
     * Input processing for the Editor
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v21) {
        switch (v21.getKeyCode()) {
            case KeyEvent.VK_UP:
                f118.m014(0, -1);
                break;
            case KeyEvent.VK_RIGHT:
                f118.m014(1, 0);
                break;
            case KeyEvent.VK_DOWN:
                f118.m014(0, +1);
                break;
            case KeyEvent.VK_LEFT:
                f118.m014(-1, 0);
                break;
            case KeyEvent.VK_ENTER:
                if (f118 == null) {
                    return;
                }

                // If not empty, bail
                if (!f318.m189(f118.m320(), f118.m420())) {
                    return;
                }

                switch (f418) {
                    case GameObject.f620:
                        f318.m69(f118.m320(), f118.m420(), f518);
                        break;
                    case GameObject.f020:
                        f318.m79(new Item(GameObject.f020, Color.WHITE, f318,
                                f118.m320(), f118.m420()));
                        break;
                    case GameObject.f120:
                        f318.m79(new Item(GameObject.f120, Color.WHITE, f318, f118
                                .m320(), f118.m420()));
                        break;
                    case GameObject.f420:
                        if (f618.equals("Blinky")) {
                            f318.m89(new Ghost(Color.RED, f318, f118.m320(), f118.m420(),
                                    f718));
                        } else if (f618.equals("Pinky")) {
                            f318.m89(new Ghost(Color.PINK, f318, f118.m320(), f118.m420(),
                                    f718));
                        } else if (f618.equals("Inky")) {
                            f318.m89(new Ghost(Color.CYAN, f318, f118.m320(), f118.m420(),
                                    f718));
                        } else {
                            f318.m89(new Ghost(Color.ORANGE, f318, f118.m320(), f118.m420(),
                                    f718));
                        }
                        break;
                    case GameObject.f320:
					int v22;
					v22 = 0;
					while (v22 < f318.m29()) {
						if (f318.m119(v22).m020() == GameObject.f320) {
							f318.m149(v22);
							v22 = v22 - 1;
						}
						v22 = v22 + 1;
					}

                        // Add the new player
                        f318.m89(new Player(f318, f118.m320(), f118.m420()));
                        break;
                    case GameObject.f720:
					final Item v23;
					v23 = new Item(GameObject.f720,
							Color.LIGHT_GRAY, f318, f118.m320(), f118.m420());
                        v23.m012(f818, f918);
                        f318.m79(v23);
                        break;

                    default:
                        break;
                }
                break;
            case KeyEvent.VK_DELETE:
                // Delete a placed object. Will reduce excessive memory
                // consumption if the user cant just replace a tile with a new
                // object

                // If empty, bail
                if (f318.m189(f118.m320(), f118.m420())) {
                    return;
                }

                // Remove anything (collidable, actor, or item) at (x,y)
                f318.m169(f118.m320(), f118.m420());
                break;
            case KeyEvent.VK_V:
                f218 = !f218;
                break;
            case KeyEvent.VK_0:
                // editorFrame.setEnabled(false);
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }

}
