package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FilenameFilter;

import pacman.game.Game;

/**
 * StateMenu is a graphical representation that allows users to switch into
 * other States of the program like the Game and editor
 * 
 * @author Ramsey Kant
 */
public class StateMenu extends State {

    // Private instance
    private int f011;
    private int f111;
    private byte f211;
    private byte f311; // Corresponds to the index in mapList
    private String[] f411;

    public StateMenu(Game v0) {
        super(v0);
    }

    @Override
    public void m13() {
        // Set cursor & menu position
        f011 = 380;
        f111 = 310;
        f211 = 0;
        f311 = 0;

        final File v1;
		v1 = new File(System.getProperty("user.dir"));

        final FilenameFilter v2;
		v2 = new FilenameFilter() {
			@Override
			public boolean accept(File v3, String v4) {
				return v4.endsWith(".map");
			}
		};

        // Apply the filter
        f411 = v1.list(v2);

        if (f411 == null) {
            System.out.println("No maps exist!");
            f63.m713(f53);
            return;
        }
    }

    /**
     * Cleanup Menu objects
     * 
     * @see State#m33()
     */
    @Override
    public void m33() {
        // do nothing
    }

    /**
     * Logic processing for the Menu. Rendering, Input, screen pointer
     * manipulation
     * 
     * @see State#m23()
     */
    @Override
    public void m23() {
        final Graphics2D v5;
		v5 = f63.m213();

        // Draw title
        v5.setColor(Color.YELLOW);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 50));
        v5.fillArc(56, 92, 100, 100, 35, 270); // First pacman
        v5.drawString("PACMAN", 350, 180);
        v5.fillArc(780, 92, 100, 100, 35, 270);

        // Draw menu options
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("Play Game", 380, 300);
        //g.drawString("Map Editor", 525, 340);
        v5.drawString("Scoreboard", 380, 340);
        v5.drawString("Exit", 380, 380);
        if (f411.length > 0) {
            v5.drawString("Current Map: " + f411[f311], 380, 600);
        } else {
            v5.drawString(
                    "No maps detected. Have you placed the maps file in the same directory as the program?",
                    100, 600);
        }

        // Draw underline cursor
        v5.setColor(Color.RED);
        v5.fillRect(f011, f111, 150, 5);
    }

    @Override
    public void keyPressed(KeyEvent v6) {
        switch (v6.getKeyCode()) {
            case KeyEvent.VK_RIGHT:
                if (f311 >= 0 && f311 < (f411.length - 1)) {
                    f311 = (byte) (f311 + 1);
                }
                break;
            case KeyEvent.VK_LEFT:
                if (f311 > 0 && f311 <= (f411.length - 1)) {
                    f311 = (byte) (f311 - 1);
                }
                break;
            case KeyEvent.VK_DOWN:
                if (f211 >= 0 && f211 < 2) {
                    f211 = (byte) (f211 + 1);
                    f111 = f111 + (38);
                }
                break;
            case KeyEvent.VK_UP:
                if (f211 > 0 && f211 <= 2) {
                    f211 = (byte) (f211 - 1);
                    f111 = f111 - (38);
                }
                break;
            case KeyEvent.VK_ENTER:
                // Execute the appropriate state change
                switch (f211) {
                    case 0:
                        // Play game
                        if (f411.length > 0) {
                            f63.m413(f411[f311]);
                            f63.m713(f23);
                        }
                        break;
                    case 1:
                        // Scoreboard
                        f63.m713(f13);
                        break;
                    case 2:
                        // Exit
                        f63.m713(f53);
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }

}
