package pacman.game;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.actors.Player;
import pacman.map.Map;

/**
 * Item objects are GameObject's that can be manipulated by the Player on the
 * map (teleports, dots, powerups, fruit) Item is a subclass of GameObject
 * 
 * @author Ramsey Kant
 */
public class Item extends GameObject {
    // Teleportation vars
    private int f012;
    private int f112;

    /**
     * Class constructor for Item
     * 
     * @param v0
     *            Object type
     * @param v1
     *            Base color of the item
     * @param v2
     *            Reference to the map object
     * @param v3
     *            X coordinate the item will occupy on the map
     * @param v4
     *            Y coordinate the item with occupy on the map
     * @see GameObject
     */
    public Item(int v0, Color v1, Map v2, int v3, int v4) {
        super(v0, v1, v2, v3, v4);

        f012 = 13;
        f112 = 17;
    }

    /**
     * Set the destination coordinates for teleportation. This isn't useful to
     * any item other than a teleport
     * 
     * @param v5
     *            X destination coordinate
     * @param v6
     *            Y destination coordinate
     */
    public void m012(int v5, int v6) {
        f012 = v5;
        f112 = v6;
    }

    /**
     * Retrieve the teleport destination X coordinate
     * 
     * @return X destination coordinate
     * @see Item#m012(int, int)
     */
    public int m112() {
        return f012;
    }

    /**
     * Retrieve the teleport destination Y coordinate
     * 
     * @return Y destination coordinate
     * @see Item#m012(int, int)
     */
    public int m212() {
        return f112;
    }

    /**
     * Called when the item is picked up / used by the player (in the player's
     * act() function) Add point values or trigger powerup modifiers here (using
     * the pl object)
     * 
     * @param v7
     *            Player that uses the item
     * @return True->Destroy the item. False->Keep the item on the map
     * @see Player#m520()
     */
    public boolean m312(Player v7) {
        boolean v8;
		v8 = false;

        // Perform action based on type
        switch (f1520) {
            case f020:
                v7.m015(10);
                v8 = true;
                break;
            case f120:
                v7.m015(50);
                v7.m315(true);
                v8 = true;
                break;
            case f720:
                v7.m76(f012, f112);
                break;
            default:
                break;
        }

        return v8;
    }

    /**
     * Item's have no "think" process. Blank method
     * 
     * @see GameObject#m520()
     */
    @Override
    public void m520() {
        // do nothing
    }

    /**
     * Draw the item based on it's type
     * 
     * @see GameObject#m620(java.awt.Graphics2D)
     */
    @Override
    public void m620(Graphics2D v9) {
        v9.setColor(f1620);

        final int v10;
		v10 = (f1720 * f1920.f29) + f1920.f29 / 2;
        final int v11;
		v11 = (f1820 * f1920.f29) + f1920.f29 / 2;

        // Render item based on type
        switch (f1520) {
            case f020:
                v9.fillArc(v10 - 4, v11 - 4, 8, 8, 0, 360);
                break;
            case f120:
                v9.fillArc(v10 - 8, v11 - 8, 16, 16, 0, 360);
                break;
            case f720:
                v9.fillOval(v10 - 6, v11 - 8, 12, 16);
                break;
            default:
                break;
        }
    }

}
