package pacman.state;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;

import pacman.actors.Actor;
import pacman.actors.Ghost;
import pacman.actors.Player;
import pacman.ai.AIManager;
import pacman.game.Game;
import pacman.game.GameObject;
import pacman.game.Item;
import pacman.map.Map;
import pacman.util.Direction;

/**
 * StateGame is a mode of the program where the user can play the game of Pacman
 * various maps StateGame is a subclass of State
 * 
 * @author Ramsey Kant
 */
public class StateGame extends State {

    // Logic object references
    private Player f08;
    private Map f18;
    private AIManager f28;

    // Game vars
    private String f38;
    private int f48;
    private int f58; // Overall score for the game session. The player
                              // object score is only the score for that life /
                              // level
    private int f68;
    private boolean f78;
    private long f88;

    // Map vars. Store them as class member vars to eliminate function call
    // overhead for getHeight/getWidth
    private int f98;
    private int f108;

    /**
     * StateGame Constructor
     * 
     * @param v0
     *            Reference to the game supervisor
     */
    public StateGame(Game v0) {
        super(v0);
    }

    /**
     * Get the current session score. A session score is the total score of the
     * entire 'game' contained by the limited number of lives This is compounded
     * at game over and in win()
     * 
     * @see StateGame#m28()
     * @see StateGame#m38()
     */
    public int m08() {
        return f58;
    }

    // Public Methods

    /**
     * Reset the state of the game entirely (level 1)
     * 
     * @see State#m13()
     */
    @Override
    public void m13() {
        // Set game vars
        f38 = f63.m313();
        f48 = 0;
        f58 = 0;
        f68 = 99;
        f88 = 0;

        // Respawn (start level 1)
        m18(true);
    }

    /**
     * Respawns the player after a death or win. Similar to reset() except
     * running vars are saved
     * 
     * @param v1
     *            Moves to the next level and corresponding map
     */
    public void m18(boolean v1) {
        f78 = true;
        f88 = System.currentTimeMillis() + 3000;

        // If we're jumping to the next level, reset everything
        if (v1) {
            f48 = f48 + 1;

            // Force previous references out of scope
            f08 = null;
            f18 = null;
            f28 = null;

            // Setup the game map
            f63.m213().setBackground(Color.BLACK);
            f18 = new Map(f38, 0.75);
            f98 = f18.m09();
            f108 = f18.m19();

            // Spawn the player
            f08 = f18.m129();

            // Setup AI
            f28 = new AIManager(f18, f08, f63.m513());

            // Slighly increase the game speed

        } else { // Player died, reset the map
            final int v2;
			v2 = f18.m29();
            int v3;
			v3 = 0;
			while (v3 < v2) {
				final Actor v4;
				v4 = f18.m119(v3);
				if (v4 != null) {
					v4.m76(v4.m06(), v4.m16());
					v4.m26(false);
					if (v4.m020() == GameObject.f420) {
						((Ghost) v4).m54(null);
					}
				}
				v3 = v3 + 1;
			}
        }
    }

    /**
     * Main game logic for rendering and processing. Called by mainThreadLoop
     * 
     * @see Game#m813()
     * @see State#m23()
     */
    @Override
    public void m23() {
        if (f18 == null) {
            return;
        }

        final Graphics2D v5;
		v5 = f63.m213();

        // Offset the buffer so object's arent clipped by the window borders
        v5.translate(10, 30);

        // Paint right UI with lives remaining, score, highscore etc
        v5.setColor(Color.WHITE);
        v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
        v5.drawString("PACMAN by Ramsey Kant", 680, 50);
        v5.drawString("Score: " + f08.m115(), 750, 100);
        v5.drawString("Total: " + f58, 750, 150);
        v5.drawString("Lives: " + f68, 750, 200);
        v5.drawString("Level: " + f48, 750, 250);

        // Execute game logic for all entites on the map
        if (!f78) {
            f28.m20();
            f08.m520();
        }

        // Check for player death. End the round if the player is dead
        if (f08.m36()) {
            m38();
            return;
        }

        // Check for a win (all dots collected)
        if (f18.m59() <= 0) {
            m28();
            return;
        }

        Item v6;
		v6 = null;
        int v7;
		v7 = 0;
		while (v7 < f98) {
			int v8;
			v8 = 0;
			while (v8 < f108) {
				final byte v9;
				v9 = f18.m99(v7, v8);
				v5.setColor(Color.BLUE);
				switch (v9) {
				case 0:
					break;
				case GameObject.f820:
					v5.fillRoundRect(v7 * f18.f29 + f18.f49, v8 * f18.f29,
							f18.f39, f18.f29, 0, 0);
					break;
				case GameObject.f920:
					v5.fillRoundRect(v7 * f18.f29, v8 * f18.f29 + f18.f49,
							f18.f29, f18.f39, 0, 0);
					break;
				case GameObject.f1020:
					v5.fillRoundRect(v7 * f18.f29 + (f18.f29 / 2), v8 * f18.f29
							+ f18.f49, f18.f29 / 2, f18.f39, 0, 0);
					v5.fillRoundRect(v7 * f18.f29 + f18.f49, v8 * f18.f29
							+ (f18.f29 / 2), f18.f39, f18.f29 / 2, 0, 0);
					break;
				case GameObject.f1120:
					v5.fillRoundRect(v7 * f18.f29, v8 * f18.f29 + f18.f49,
							f18.f29 / 2, f18.f39, 0, 0);
					v5.fillRoundRect(v7 * f18.f29 + f18.f49, v8 * f18.f29
							+ (f18.f29 / 2), f18.f39, f18.f29 / 2, 0, 0);
					break;
				case GameObject.f1220:
					v5.fillRoundRect(v7 * f18.f29 + (f18.f29 / 2), v8 * f18.f29
							+ f18.f49, f18.f29 / 2, f18.f39, 0, 0);
					v5.fillRoundRect(v7 * f18.f29 + f18.f49, v8 * f18.f29, 12,
							f18.f29 / 2, 0, 0);
					break;
				case GameObject.f1320:
					v5.fillRoundRect(v7 * f18.f29, v8 * f18.f29 + f18.f49,
							f18.f29 / 2, f18.f39, 0, 0);
					v5.fillRoundRect(v7 * f18.f29 + f18.f49, v8 * f18.f29,
							f18.f39, f18.f29 / 2, 0, 0);
					break;
				case GameObject.f1420:
					v5.setColor(Color.PINK);
					v5.fillRoundRect(v7 * f18.f29, v8 * f18.f29 + f18.f49,
							f18.f29, f18.f39 / 2, 0, 0);
					break;
				default:
					break;
				}
				v6 = f18.m109(v7, v8);
				if (v6 != null) {
					v6.m620(v5);
				}
				v8 = v8 + 1;
			}
			v7 = v7 + 1;
		}

        final int v10;
		v10 = f18.m29();
        int v11;
		v11 = 0;
		while (v11 < v10) {
			final Actor v12;
			v12 = f18.m119(v11);
			if (v12 != null) {
				v12.m620(v5);
			}
			v11 = v11 + 1;
		}

        // Debug
        if (f63.m513()) {
            v5.setColor(Color.RED);
            v5.drawString("DEBUG ON", 750, 650);
            /*
             * // Paint gridline overlay for(int i = 0; i < mapWidth; i++)
             * g.drawLine(i*map.CELL_SIZE, 0, i*map.CELL_SIZE,
             * mapHeight*map.CELL_SIZE); for(int i = 0; i < mapHeight; i++)
             * g.drawLine(0, i*map.CELL_SIZE, mapWidth*map.CELL_SIZE,
             * i*map.CELL_SIZE);
             */

            // Player X,Y coordinates bottom right
            v5.drawString("positionX: " + f08.m320(), 750, 675);
            v5.drawString("positionY: " + f08.m420(), 750, 700);
        }

        // Check for game pause and print pause status
        if (f78) {
            v5.setColor(Color.RED);
            v5.setFont(new Font("Comic Sans MS", Font.BOLD, 24));
            v5.drawString("PAUSED", 750, 500);
            if (f88 > System.currentTimeMillis()) {
                v5.drawString(
                        "Pause ends in..." + ((f88 - System.currentTimeMillis()) / 1000),
                        750, 550);
            }
            if (f88 != 0 && System.currentTimeMillis() > f88) {
                f88 = 0;
                f78 = false;
            }
            return;
        }
    }

    /**
     * Player has died or the Supervisor has decided to change the state
     * abruptly
     * 
     * @see State#m33()
     */
    @Override
    public void m33() {
        // Cleanup
        f08 = null;
        f18 = null;
    }

    /**
     * Player has won, move to the next level Called by logic()
     * 
     * @see StateGame#m23()
     */
    public void m28() {
        f58 = f58 + (f08.m115());

        m18(true);
    }

    /**
     * Player has died, reset() if lives remain. Otherwise, request a state
     * change thereby end()ing this state Called by logic()
     * 
     * @see StateGame#m23()
     */
    public void m38() {
        f68 = f68 - 1;

        if (f68 > 0) {
            m18(false);
        } else {
            if (f48 == 1) {
                f58 = f08.m115(); // win() never called, so
                                                  // score is the 1st level
                                                  // score
            }
            f63.m713(State.f13);
        }
    }

    /**
     * Start automove in certain direction
     * 
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    @Override
    public void keyPressed(KeyEvent v13) {
        if (f08 == null) {
            return;
        }

        switch (v13.getKeyCode()) {
            case KeyEvent.VK_UP:
                f08.m66(Direction.f07);
                break;
            case KeyEvent.VK_RIGHT:
                f08.m66(Direction.f17);
                break;
            case KeyEvent.VK_DOWN:
                f08.m66(Direction.f27);
                break;
            case KeyEvent.VK_LEFT:
                f08.m66(Direction.f37);
                break;
            case KeyEvent.VK_SPACE:
                f08.m66(Direction.f57);
                break;
            case KeyEvent.VK_P:
                // Don't interupt system pauses
                if (f88 < System.currentTimeMillis()) {
                    f78 = !f78;
                }
                break;
            case KeyEvent.VK_V:
                f63.m613();
                // AI debug
                f28.m00(f63.m513());
                break;
            case KeyEvent.VK_0:
                // game.changeState(STATE_MENU);
                break;
            default:
                break;
        }
    }
}
