package pacman.game;

import java.awt.Color;
import java.awt.Graphics2D;

import pacman.actors.Actor;
import pacman.map.Map;
import pacman.state.StateGame;

/**
 * A game object is anything on the pacman grid (wall, cherry, ghost, player).
 * GameObject is the base class of almost everything within the Map
 * 
 * @author Ramsey Kant
 */
public abstract class GameObject {
    // Static type vars
    public static final int f020 = 1;
    public static final int f120 = 2;
    public static final int f220 = 4;
    public static final int f320 = 8;
    public static final int f420 = 16;
    public static final int f520 = 32; // Virtual
    public static final int f620 = 64; // Virtual
    public static final int f720 = 128;

    // Wall types (Walls aren't instanced GameObject's)
    public static final byte f820 = 1;
    public static final byte f920 = 2;
    public static final byte f1020 = 3;
    public static final byte f1120 = 4;
    public static final byte f1220 = 5;
    public static final byte f1320 = 6;
    public static final byte f1420 = 7;

    // Generic object attributes
    protected int f1520;
    protected Color f1620;
    protected int f1720;
    protected int f1820;

    // Outside refereneces
    protected final Map f1920; // Can only be set once. Object only exists within
                             // the map. If the map changes, new objects are
                             // created

    // Getters and Setters

    /**
     * Return the type of the object set in the constructor. See static types
     * defined in GameObject
     * 
     * @return type of object
     */
    public int m020() {
        return f1520;
    }

    /**
     * Grab the current java.awt.Color (base color) the object is being rendered
     * in
     * 
     * @return Base Color of the object
     */
    public Color m120() {
        return f1620;
    }

    /**
     * Set the current base color used when rendering the object
     * 
     * @param v0
     *            java.awt.Color Color of object
     */
    public void m220(Color v0) {
        f1620 = v0;
    }

    /**
     * Grab the current X coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m76(int, int)
     */
    public int m320() {
        return f1720;
    }

    /**
     * Grab the current Y coordinate of the object on the map. This property is
     * frequently modified by the Map class and move() method
     * 
     * @see Actor#m76(int, int)
     */
    public int m420() {
        return f1820;
    }

    // Public & Protected Abstract methods

    /**
     * Class Constructor for a game object
     * 
     * @param v1
     *            Type of game object (see static types above)
     * @param v2
     *            Standard java Color
     * @param v3
     *            Reference to the global Map
     * @param v4
     *            Initial x coordinate
     * @param v5
     *            Initial y coordinate
     */
    public GameObject(int v1, Color v2, Map v3, int v4, int v5) {
        f1520 = v1;
        f1620 = v2;
        f1920 = v3;
        f1720 = v4;
        f1820 = v5;
    }

    /**
     * Perform a "Think" cycle for the Object This includes things like self
     * maintenance and movement
     */
    public abstract void m520();

    /**
     * Draw the object. Subclasses should define how they are to be drawn. This
     * is called in StateGame's logic()
     * 
     * @param v6
     *            The graphics context
     * @see StateGame#m23()
     */
    public abstract void m620(Graphics2D v6);
}
